#!/usr/bin/env python

# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies duplicate ldflags are not removed.
"""

from __future__ import print_function

import sys

def CheckContainsFlags(args, substring):
  if args.find(substring) is -1:
    print('ERROR: Linker arguments "%s" are missing in "%s"' % (substring,
                                                                args))
    return False;
  return True;

if __name__ == '__main__':
  args = " ".join(sys.argv)
  print("args = " +args)
  if not CheckContainsFlags(args, 'lib1.a -Wl,--no-whole-archive') \
    or not CheckContainsFlags(args, 'lib2.a -Wl,--no-whole-archive'):
    sys.exit(1);
  sys.exit(0)



#!/usr/bin/env python
#
# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import os
import sys

"""A slightly odd 'cp' implementation for this test.

This 'cp' can have many targets, but only one source. 'cp src dest1 dest2'
will copy the file 'src' to both 'dest1' and 'dest2'."""

with open(sys.argv[1], 'r') as f:
  src = f.read()
for dest in sys.argv[2:]:
  with open(dest, 'w') as f:
    f.write(src)




#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that IPHONEOS_DEPLOYMENT_TARGET works.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['make', 'ninja', 'xcode'])

  test.run_gyp('deployment-target.gyp', chdir='deployment-target')

  test.build('deployment-target.gyp', test.ALL, chdir='deployment-target')

  test.pass_test()




#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies file copies where two copies sections in the same target have the
same destination directory.
"""

import TestGyp

test = TestGyp.TestGyp()
test.run_gyp('copies-samedir.gyp', chdir='src')

test.relocate('src', 'relocate/src')
test.build('copies-samedir.gyp', 'copies_samedir', chdir='relocate/src')

test.built_file_must_match('copies-out-samedir/file1',
                           'file1 contents\n',
                           chdir='relocate/src')

test.built_file_must_match('copies-out-samedir/file2',
                           'file2 contents\n',
                           chdir='relocate/src')

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2009 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies copies in sourceless shared_library targets are executed.
"""

import TestGyp

test = TestGyp.TestGyp()
test.run_gyp('copies-sourceless-shared-lib.gyp', chdir='src')
test.relocate('src', 'relocate/src')
test.build('copies-sourceless-shared-lib.gyp', chdir='relocate/src')
test.built_file_must_match('copies-out/file1',
                           'file1 contents\n',
                           chdir='relocate/src')
test.pass_test()



#!/usr/bin/env python

# Copyright 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that iOS XCTests can be built correctly.
"""

import TestGyp

import os
import subprocess
import sys

def HasCerts():
  # Because the bots do not have certs, don't check them if there are no
  # certs available.
  proc = subprocess.Popen(['security','find-identity','-p', 'codesigning',
                           '-v'], stdout=subprocess.PIPE)
  return "0 valid identities found" not in proc.communicate()[0].strip()

if sys.platform == "darwin":
  # This test appears to be flaky and hangs some of the time.
  sys.exit(2)  # bug=531

  test = TestGyp.TestGyp(formats=['xcode', 'ninja'])
  test.run_gyp('xctests.gyp')
  test_configs = ['Default']
  # TODO(crbug.com/557418): Enable this once xcodebuild works for iOS devices.
  #if HasCerts() and test.format == 'xcode':
  #  test_configs.append('Default-iphoneos')
  for config in test_configs:
    test.set_configuration(config)
    test.build('xctests.gyp', test.ALL)
    test.built_file_must_exist('app_under_test.app/app_under_test')
    test.built_file_must_exist('app_tests.xctest/app_tests')
    if 'ninja' in test.format:
      test.built_file_must_exist('obj/AppTests/app_tests.AppTests.i386.o')
      test.built_file_must_exist('obj/AppTests/app_tests.AppTests.x86_64.o')
    elif test.format == 'xcode':
      xcode_object_path = os.path.join('..', 'xctests.build',
                                       'Default-iphonesimulator',
                                       'app_tests.build', 'Objects-normal',
                                       '%s', 'AppTests.o')
      test.built_file_must_exist(xcode_object_path % 'i386')
      test.built_file_must_exist(xcode_object_path % 'x86_64')
  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure macro expansion of $(TargetFileName) is handled.
"""

from __future__ import print_function

import TestGyp

import os
import sys

if sys.platform == 'win32':
  print("This test is currently disabled: https://crbug.com/483696.")
  sys.exit(0)

  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])
  if not (test.format == 'msvs' and
          int(os.environ.get('GYP_MSVS_VERSION', 0)) == 2013):
    CHDIR = 'vs-macros'
    test.run_gyp('targetfilename.gyp', chdir=CHDIR)
    test.build('targetfilename.gyp', test.ALL, chdir=CHDIR)
    test.built_file_must_exist('test_targetfilename_executable.exe', chdir=CHDIR)
    test.built_file_must_exist('test_targetfilename_loadable_module.dll',
                              chdir=CHDIR)
    test.built_file_must_exist('test_targetfilename_shared_library.dll',
                              chdir=CHDIR)
    test.built_file_must_exist('test_targetfilename_static_library.lib',
                              chdir=CHDIR)
    test.built_file_must_exist('test_targetfilename_product_extension.foo',
                              chdir=CHDIR)
    test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure msvs_large_pdb works correctly.
"""

from __future__ import print_function

import TestGyp

import struct
import sys

if sys.platform == 'win32':
  print("This test is currently disabled: https://crbug.com/483696.")
  sys.exit(0)


CHDIR = 'large-pdb'


def CheckImageAndPdb(test, image_basename, expected_page_size,
                     pdb_basename=None):
  if not pdb_basename:
    pdb_basename = image_basename + '.pdb'
  test.built_file_must_exist(image_basename, chdir=CHDIR)
  test.built_file_must_exist(pdb_basename, chdir=CHDIR)

  # We expect the PDB to have the given page size. For full details of the
  # header look here: https://code.google.com/p/pdbparser/wiki/MSF_Format
  # We read the little-endian 4-byte unsigned integer at position 32 of the
  # file.
  pdb_path = test.built_file_path(pdb_basename, chdir=CHDIR)
  pdb_file = open(pdb_path, 'rb')
  pdb_file.seek(32, 0)
  page_size = struct.unpack('<I', pdb_file.read(4))[0]
  if page_size != expected_page_size:
    print("Expected page size of %d, got %d for PDB file `%s'." % (
        expected_page_size, page_size, pdb_path))


if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  test.run_gyp('large-pdb.gyp', chdir=CHDIR)

  test.build('large-pdb.gyp', 'large_pdb_exe', chdir=CHDIR)
  CheckImageAndPdb(test, 'large_pdb_exe.exe', 4096)

  test.build('large-pdb.gyp', 'small_pdb_exe', chdir=CHDIR)
  CheckImageAndPdb(test, 'small_pdb_exe.exe', 1024)

  test.build('large-pdb.gyp', 'large_pdb_dll', chdir=CHDIR)
  CheckImageAndPdb(test, 'large_pdb_dll.dll', 4096)

  test.build('large-pdb.gyp', 'small_pdb_dll', chdir=CHDIR)
  CheckImageAndPdb(test, 'small_pdb_dll.dll', 1024)

  test.build('large-pdb.gyp', 'large_pdb_implicit_exe', chdir=CHDIR)
  CheckImageAndPdb(test, 'large_pdb_implicit_exe.exe', 4096)

  # This target has a different PDB name because it uses an
  # 'msvs_large_pdb_path' variable.
  test.build('large-pdb.gyp', 'large_pdb_variable_exe', chdir=CHDIR)
  CheckImageAndPdb(test, 'large_pdb_variable_exe.exe', 4096,
                   pdb_basename='foo.pdb')

  # This target has a different output name because it uses 'product_name'.
  test.build('large-pdb.gyp', 'large_pdb_product_exe', chdir=CHDIR)
  CheckImageAndPdb(test, 'bar.exe', 4096)

  test.pass_test()



#!/usr/bin/env python
# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

from __future__ import print_function
print("This will go to stdout")



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

from __future__ import print_function

from optparse import OptionParser

parser = OptionParser()
parser.add_option('-a', dest='platform')
parser.add_option('-o', dest='output')
parser.add_option('-p', dest='path')
(options, args) = parser.parse_args()

f = open(options.output, 'w')
print('options', options, file=f)
print('args', args, file=f)
f.close()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure PGO is working properly.
"""

import TestGyp

import os
import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'linker-flags'
  test.run_gyp('pgo.gyp', chdir=CHDIR)

  def IsPGOAvailable():
    """Returns true if the Visual Studio available here supports PGO."""
    test.build('pgo.gyp', 'gen_linker_option', chdir=CHDIR)
    tmpfile = test.read(test.built_file_path('linker_options.txt', chdir=CHDIR))
    return any(line.find('PGOPTIMIZE') for line in tmpfile)

  # Test generated build files look fine.
  if test.format == 'ninja':
    ninja = test.built_file_path('obj/test_pgo_instrument.ninja', chdir=CHDIR)
    test.must_contain(ninja, '/LTCG:PGINSTRUMENT')
    test.must_contain(ninja, 'test_pgo.pgd')
    ninja = test.built_file_path('obj/test_pgo_optimize.ninja', chdir=CHDIR)
    test.must_contain(ninja, '/LTCG:PGOPTIMIZE')
    test.must_contain(ninja, 'test_pgo.pgd')
    ninja = test.built_file_path('obj/test_pgo_update.ninja', chdir=CHDIR)
    test.must_contain(ninja, '/LTCG:PGUPDATE')
    test.must_contain(ninja, 'test_pgo.pgd')
  elif test.format == 'msvs':
    LTCG_FORMAT = '<LinkTimeCodeGeneration>%s</LinkTimeCodeGeneration>'
    vcproj = test.workpath('linker-flags/test_pgo_instrument.vcxproj')
    test.must_contain(vcproj, LTCG_FORMAT % 'PGInstrument')
    test.must_contain(vcproj, 'test_pgo.pgd')
    vcproj = test.workpath('linker-flags/test_pgo_optimize.vcxproj')
    test.must_contain(vcproj, LTCG_FORMAT % 'PGOptimization')
    test.must_contain(vcproj, 'test_pgo.pgd')
    vcproj = test.workpath('linker-flags/test_pgo_update.vcxproj')
    test.must_contain(vcproj, LTCG_FORMAT % 'PGUpdate')
    test.must_contain(vcproj, 'test_pgo.pgd')

  # When PGO is available, try building binaries with PGO.
  if IsPGOAvailable():
    pgd_path = test.built_file_path('test_pgo.pgd', chdir=CHDIR)

    # Test if 'PGInstrument' generates PGD (Profile-Guided Database) file.
    if os.path.exists(pgd_path):
      test.unlink(pgd_path)
    test.must_not_exist(pgd_path)
    test.build('pgo.gyp', 'test_pgo_instrument', chdir=CHDIR)
    test.must_exist(pgd_path)

    # Test if 'PGOptimize' works well
    test.build('pgo.gyp', 'test_pgo_optimize', chdir=CHDIR)
    test.must_contain_any_line(test.stdout(), ['profiled functions'])

    # Test if 'PGUpdate' works well
    test.build('pgo.gyp', 'test_pgo_update', chdir=CHDIR)
    # With 'PGUpdate', linker should not complain that sources are changed after
    # the previous training run.
    test.touch(test.workpath('linker-flags/inline_test_main.cc'))
    test.unlink(test.built_file_path('test_pgo_update.exe', chdir=CHDIR))
    test.build('pgo.gyp', 'test_pgo_update', chdir=CHDIR)
    test.must_contain_any_line(test.stdout(), ['profiled functions'])

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure msvs_target_platform_version works correctly.
"""

from __future__ import print_function

import TestGyp

import os
import sys
import struct

CHDIR = 'winrt-target-platform-version'

print('This test is not currently working on the bots: https://code.google.com/p/gyp/issues/detail?id=466')
sys.exit(0)

if (sys.platform == 'win32' and
    int(os.environ.get('GYP_MSVS_VERSION', 0)) == 2015):
  test = TestGyp.TestGyp(formats=['msvs'])

  test.run_gyp('winrt-target-platform-version.gyp', chdir=CHDIR)

  test.build('winrt-target-platform-version.gyp',
             'enable_winrt_10_platversion_dll', chdir=CHDIR)

  # Target Platform without Minimum Target Platform version defaults to a valid
  # Target Platform and compiles.
  test.build('winrt-target-platform-version.gyp',
             'enable_winrt_10_platversion_nominver_dll', chdir=CHDIR)

  # Target Platform is set to 9.0 which is invalid for 2015 projects so
  # compilation must fail.
  test.build('winrt-target-platform-version.gyp',
             'enable_winrt_9_platversion_dll', chdir=CHDIR, status=1)

  # Missing Target Platform for 2015 projects must fail.
  test.build('winrt-target-platform-version.gyp',
             'enable_winrt_missing_platversion_dll', chdir=CHDIR, status=1)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies the use of the environment during regeneration when the gyp file
changes, specifically via build of an executable with C++ flags specified by
CXXFLAGS.

In this test, gyp happens within a local environment, but build outside of it.
"""

import TestGyp

FORMATS = ('ninja',)

test = TestGyp.TestGyp(formats=FORMATS)

# We reset the environ after calling gyp. When the auto-regeneration happens,
# the same define should be reused anyway.
with TestGyp.LocalEnv({'CXXFLAGS': ''}):
  test.run_gyp('cxxflags.gyp')

test.build('cxxflags.gyp')

expect = """\
No define
"""
test.run_built_executable('cxxflags', stdout=expect)

test.sleep()

with TestGyp.LocalEnv({'CXXFLAGS': '-DABC'}):
  test.run_gyp('cxxflags.gyp')

test.build('cxxflags.gyp')

expect = """\
With define
"""
test.run_built_executable('cxxflags', stdout=expect)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure the /safeseh option can be passed to ml.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['ninja'])

  CHDIR = 'ml-safeseh'
  test.run_gyp('ml-safeseh.gyp', chdir=CHDIR)
  test.build('ml-safeseh.gyp', test.ALL, chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that ARC objc settings are handled correctly.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  # set |match| to ignore build stderr output.
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'],
                         match = lambda a, b: True)

  CHDIR = 'objc-arc'
  test.run_gyp('test.gyp', chdir=CHDIR)

  test.build('test.gyp', 'arc_enabled', chdir=CHDIR)
  test.build('test.gyp', 'weak_enabled', chdir=CHDIR)
  test.build('test.gyp', 'arc_disabled', chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that CLANG_CXX_LANGUAGE_STANDARD works.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['make', 'ninja', 'xcode'])

  test.run_gyp('clang-cxx-language-standard.gyp',
               chdir='clang-cxx-language-standard')

  test.build('clang-cxx-language-standard.gyp', test.ALL,
             chdir='clang-cxx-language-standard')

  test.pass_test()




#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that msvs_external_builder being set will invoke the provided
msvs_external_builder_build_cmd and msvs_external_builder_clean_cmd, and will
not invoke MSBuild actions and rules.
"""

import os
import sys
import TestGyp

if int(os.environ.get('GYP_MSVS_VERSION', 0)) < 2010:
  sys.exit(0)

test = TestGyp.TestGyp(formats=['msvs'], workdir='workarea_all')

# without the flag set
test.run_gyp('external.gyp')
test.build('external.gyp', target='external')
test.must_not_exist('external_builder.out')
test.must_exist('msbuild_rule.out')
test.must_exist('msbuild_action.out')
test.must_match('msbuild_rule.out', 'msbuild_rule.py hello.z a b c')
test.must_match('msbuild_action.out', 'msbuild_action.py x y z')
os.remove('msbuild_rule.out')
os.remove('msbuild_action.out')

# with the flag set, using Build
try:
  os.environ['GYP_DEFINES'] = 'use_external_builder=1'
  test.run_gyp('external.gyp')
  test.build('external.gyp', target='external')
finally:
  del os.environ['GYP_DEFINES']
test.must_not_exist('msbuild_rule.out')
test.must_not_exist('msbuild_action.out')
test.must_exist('external_builder.out')
test.must_match('external_builder.out', 'external_builder.py build 1 2 3')
os.remove('external_builder.out')

# with the flag set, using Clean
try:
  os.environ['GYP_DEFINES'] = 'use_external_builder=1'
  test.run_gyp('external.gyp')
  test.build('external.gyp', target='external', clean=True)
finally:
  del os.environ['GYP_DEFINES']
test.must_not_exist('msbuild_rule.out')
test.must_not_exist('msbuild_action.out')
test.must_exist('external_builder.out')
test.must_match('external_builder.out', 'external_builder.py clean 4 5')
os.remove('external_builder.out')

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""Verifies that depfile fields are output in ninja rules."""

import TestGyp
import os

test = TestGyp.TestGyp()

if test.format == 'ninja':
  test.run_gyp('depfile.gyp')
  contents = open(test.built_file_path('obj/depfile_target.ninja')).read()

  expected = [
    'depfile = depfile_action.d',
    'depfile = ' + os.path.join(
      'obj', 'depfile_target.gen/depfile_action_intermediate_dir.d'),
  ]
  test.must_contain_all_lines(contents, expected)

  test.build('depfile.gyp')
  test.built_file_must_exist('depfile_action.d')
  test.built_file_must_exist(
    'obj/depfile_target.gen/depfile_action_intermediate_dir.d')

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies the use of the environment during regeneration when the gyp file
changes, specifically via build of an executable with C preprocessor
definition specified by CFLAGS.

In this test, gyp and build both run in same local environment.
"""

import TestGyp

# CPPFLAGS works in ninja but not make; CFLAGS works in both
FORMATS = ('make', 'ninja')

test = TestGyp.TestGyp(formats=FORMATS)

# First set CFLAGS to blank in case the platform doesn't support unsetenv.
with TestGyp.LocalEnv({'CFLAGS': '',
                       'GYP_CROSSCOMPILE': '1'}):
  test.run_gyp('cflags.gyp')
  test.build('cflags.gyp')

expect = """FOO not defined\n"""
test.run_built_executable('cflags', stdout=expect)
test.run_built_executable('cflags_host', stdout=expect)

test.sleep()

with TestGyp.LocalEnv({'CFLAGS': '-DFOO=1',
                       'GYP_CROSSCOMPILE': '1'}):
  test.run_gyp('cflags.gyp')
  test.build('cflags.gyp')

expect = """FOO defined\n"""
test.run_built_executable('cflags', stdout=expect)

# Environment variable CFLAGS shouldn't influence the flags for the host.
expect = """FOO not defined\n"""
test.run_built_executable('cflags_host', stdout=expect)

test.sleep()

with TestGyp.LocalEnv({'CFLAGS_host': '-DFOO=1',
                       'GYP_CROSSCOMPILE': '1'}):
  test.run_gyp('cflags.gyp')
  test.build('cflags.gyp')

# Environment variable CFLAGS_host should influence the flags for the host.
expect = """FOO defined\n"""
test.run_built_executable('cflags_host', stdout=expect)

test.sleep()

with TestGyp.LocalEnv({'CFLAGS': ''}):
  test.run_gyp('cflags.gyp')
  test.build('cflags.gyp')

expect = """FOO not defined\n"""
test.run_built_executable('cflags', stdout=expect)

test.sleep()

with TestGyp.LocalEnv({'CFLAGS': '-DFOO=1'}):
  test.run_gyp('cflags.gyp')
  test.build('cflags.gyp')

expect = """FOO defined\n"""
test.run_built_executable('cflags', stdout=expect)

test.pass_test()



# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
TestMac.py:  a collection of helper function shared between test on Mac OS X.
"""

from __future__ import print_function

import re
import subprocess

__all__ = ['Xcode', 'CheckFileType']


def CheckFileType(test, file, archs):
  """Check that |file| contains exactly |archs| or fails |test|."""
  proc = subprocess.Popen(['lipo', '-info', file], stdout=subprocess.PIPE)
  o = proc.communicate()[0].decode('utf-8').strip()
  assert not proc.returncode
  if len(archs) == 1:
    pattern = re.compile('^Non-fat file: (.*) is architecture: (.*)$')
  else:
    pattern = re.compile('^Architectures in the fat file: (.*) are: (.*)$')
  match = pattern.match(o)
  if match is None:
    print('Ouput does not match expected pattern: %s' % (pattern.pattern))
    test.fail_test()
  else:
    found_file, found_archs = match.groups()
    if found_file != file or set(found_archs.split()) != set(archs):
      print('Expected file %s with arch %s, got %s with arch %s' % (
          file, ' '.join(archs), found_file, found_archs))
      test.fail_test()


class XcodeInfo(object):
  """Simplify access to Xcode informations."""

  def __init__(self):
    self._cache = {}

  def _XcodeVersion(self):
    lines = subprocess.check_output(['xcodebuild', '-version']).splitlines()
    version = ''.join(lines[0].decode('utf-8').split()[-1].split('.'))
    version = (version + '0' * (3 - len(version))).zfill(4)
    return version, lines[-1].split()[-1]

  def Version(self):
    if 'Version' not in self._cache:
      self._cache['Version'], self._cache['Build'] = self._XcodeVersion()
    return self._cache['Version']

  def Build(self):
    if 'Build' not in self._cache:
      self._cache['Version'], self._cache['Build'] = self._XcodeVersion()
    return self._cache['Build']

  def SDKBuild(self):
    if 'SDKBuild' not in self._cache:
      self._cache['SDKBuild'] = subprocess.check_output(
          ['xcodebuild', '-version', '-sdk', '', 'ProductBuildVersion'])
      self._cache['SDKBuild'] = self._cache['SDKBuild'].decode('utf-8')
      self._cache['SDKBuild'] = self._cache['SDKBuild'].rstrip('\n')
    return self._cache['SDKBuild']

  def SDKVersion(self):
    if 'SDKVersion' not in self._cache:
      self._cache['SDKVersion'] = subprocess.check_output(
          ['xcodebuild', '-version', '-sdk', '', 'SDKVersion'])
      self._cache['SDKVersion'] = self._cache['SDKVersion'].rstrip('\n')
    return self._cache['SDKVersion']


Xcode = XcodeInfo()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Checks that gyp fails on static_library targets which have several files with
the same basename.
"""

import os
import sys

import TestGyp

test = TestGyp.TestGyp()

# Fails by default for the compatibility with legacy generators such as
# VCProj generator for Visual C++ 2008 and Makefile generator on Mac.
# TODO: Update expected behavior when these legacy generators are deprecated.
test.run_gyp('double-static.gyp', chdir='src', status=1, stderr=None)

if ((test.format == 'make' and sys.platform == 'darwin') or
    (test.format == 'msvs' and
        int(os.environ.get('GYP_MSVS_VERSION', 2010)) < 2010)):
  test.run_gyp('double-static.gyp', '--no-duplicate-basename-check',
               chdir='src', status=1, stderr=None)
else:
  test.run_gyp('double-static.gyp', '--no-duplicate-basename-check',
               chdir='src')
  test.build('double-static.gyp', test.ALL, chdir='src')

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure that when ULDI is on, we link cause downstream modules to get built
when we depend on the component objs.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'uldi'
  test.run_gyp('uldi-depending-on-module.gyp', chdir=CHDIR)
  test.build('uldi-depending-on-module.gyp', 'an_exe', chdir=CHDIR)
  test.built_file_must_exist('a_module.dll', chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2016 Mark Callow. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that files are copied to the correct destinations when those
destinations are specified using environment variables available in
Xcode's PBXCopyFilesBuildPhase.
"""

import TestGyp

import os
import stat
import sys


test = TestGyp.TestGyp(formats=['make', 'ninja', 'xcode'])

if sys.platform == 'darwin':
  test.run_gyp('copies-with-xcode-envvars.gyp',
               chdir='copies-with-xcode-envvars')

  test.build('copies-with-xcode-envvars.gyp', chdir='copies-with-xcode-envvars')

  wrapper_name = 'copies-with-xcode-envvars.app/'
  contents_path = wrapper_name + 'Contents/'
  out_path = test.built_file_path('file0', chdir='copies-with-xcode-envvars')
  test.must_contain(out_path, 'file0 contents\n')
  out_path = test.built_file_path(wrapper_name + 'file1',
                                  chdir='copies-with-xcode-envvars')
  test.must_contain(out_path, 'file1 contents\n')
  out_path = test.built_file_path(contents_path + 'MacOS/file2',
  chdir='copies-with-xcode-envvars')
  test.must_contain(out_path, 'file2 contents\n')
  out_path = test.built_file_path(contents_path + 'Resources/file3',
                                  chdir='copies-with-xcode-envvars')
  test.must_contain(out_path, 'file3 contents\n')
  out_path = test.built_file_path(contents_path + 'Resources/testimages/file4',
                                  chdir='copies-with-xcode-envvars')
  test.must_contain(out_path, 'file4 contents\n')
  out_path = test.built_file_path(contents_path + 'Resources/Java/file5',
                                  chdir='copies-with-xcode-envvars')
  test.must_contain(out_path, 'file5 contents\n')
  out_path = test.built_file_path(contents_path + 'Frameworks/file6',
                                  chdir='copies-with-xcode-envvars')
  test.must_contain(out_path, 'file6 contents\n')
  out_path = test.built_file_path(contents_path + 'Frameworks/file7',
                                  chdir='copies-with-xcode-envvars')
  test.must_contain(out_path, 'file7 contents\n')
  out_path = test.built_file_path(contents_path + 'SharedFrameworks/file8',
                                  chdir='copies-with-xcode-envvars')
  test.must_contain(out_path, 'file8 contents\n')
  out_path = test.built_file_path(contents_path + 'SharedSupport/file9',
                                  chdir='copies-with-xcode-envvars')
  test.must_contain(out_path, 'file9 contents\n')
  out_path = test.built_file_path(contents_path + 'PlugIns/file10',
                                  chdir='copies-with-xcode-envvars')
  test.must_contain(out_path, 'file10 contents\n')
  out_path = test.built_file_path(contents_path + 'XPCServices/file11',
                                  chdir='copies-with-xcode-envvars')
  test.must_contain(out_path, 'file11 contents\n')
  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure StackReserveSize and StackCommitSize settings are extracted properly.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'linker-flags'
  test.run_gyp('stacksize.gyp', chdir=CHDIR)
  test.build('stacksize.gyp', test.ALL, chdir=CHDIR)

  def GetHeaders(exe):
    return test.run_dumpbin('/headers', test.built_file_path(exe, chdir=CHDIR))

  # Verify default sizes as reported by dumpbin:
  #     100000h = 1MB
  #     1000h   = 4KB
  default_headers = GetHeaders('test_default.exe')
  if '100000 size of stack reserve' not in default_headers:
    test.fail_test()
  if '1000 size of stack commit' not in default_headers:
    test.fail_test()

  # Verify that reserved size is changed, but commit size is unchanged:
  #     200000h = 2MB
  #     1000h   = 4KB
  set_reserved_size_headers = GetHeaders('test_set_reserved_size.exe')
  if '200000 size of stack reserve' not in set_reserved_size_headers:
    test.fail_test()
  if '1000 size of stack commit' not in set_reserved_size_headers:
    test.fail_test()

  # Verify that setting the commit size, without the reserve size, has no
  # effect:
  #     100000h = 1MB
  #     1000h   = 4KB
  set_commit_size_headers = GetHeaders('test_set_commit_size.exe')
  if '100000 size of stack reserve' not in set_commit_size_headers:
    test.fail_test()
  if '1000 size of stack commit' not in set_commit_size_headers:
    test.fail_test()

  # Verify that setting both works:
  #     200000h = 2MB
  #     2000h   = 8KB
  set_both_headers = GetHeaders('test_set_both.exe')
  if '200000 size of stack reserve' not in set_both_headers:
    test.fail_test()
  if '2000 size of stack commit' not in set_both_headers:
    test.fail_test()

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure msvs_shard works correctly.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'shard'
  test.run_gyp('shard.gyp', chdir=CHDIR)
  test.build('shard.gyp', test.ALL, chdir=CHDIR)

  test.built_file_must_exist('shard_0.lib', chdir=CHDIR)
  test.built_file_must_exist('shard_1.lib', chdir=CHDIR)
  test.built_file_must_exist('shard_2.lib', chdir=CHDIR)
  test.built_file_must_exist('shard_3.lib', chdir=CHDIR)

  test.run_gyp('shard_ref.gyp', chdir=CHDIR)
  test.build('shard_ref.gyp', test.ALL, chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies building a target with nothing succeeds.
"""

import os
import sys
import TestGyp

test = TestGyp.TestGyp()
test.run_gyp('empty-target.gyp')
test.build('empty-target.gyp', target='empty_target')
test.pass_test()



# Copyright (c) 2017 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

from __future__ import print_function

print("sample\\path\\foo.cpp")



#!/usr/bin/env python

# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure that the (custom) NoImportLibrary flag is handled correctly.
"""

import TestGyp

import os
import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['ninja'])

  CHDIR = 'importlib'
  test.run_gyp('noimplib.gyp', chdir=CHDIR)
  test.build('noimplib.gyp', test.ALL, chdir=CHDIR)

  # The target has an entry point, but no exports. Ordinarily, ninja expects
  # all DLLs to export some symbols (with the exception of /NOENTRY resource-
  # only DLLs). When the NoImportLibrary flag is set, this is suppressed. If
  # this is not working correctly, the expected .lib will never be generated
  # but will be expected, so the build will not be up to date.
  test.up_to_date('noimplib.gyp', test.ALL, chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that ios app bundles are built correctly.
"""

import TestGyp
import TestMac

import os.path
import sys

# Xcode supports for assets catalog was introduced in Xcode 6.0
if sys.platform == 'darwin' and TestMac.Xcode.Version() >= '0600':
  test_gyp_path = 'test-assets-catalog.gyp'
  test_app_path = 'Test App Assets Catalog Gyp.app'

  test = TestGyp.TestGyp(formats=['xcode', 'ninja'])
  test.run_gyp(test_gyp_path, chdir='app-bundle')
  test.build(test_gyp_path, test.ALL, chdir='app-bundle')

  # Test that the extension is .bundle
  test.built_file_must_exist(
      os.path.join(test_app_path, 'Test App Assets Catalog Gyp'),
      chdir='app-bundle')

  # Info.plist
  info_plist = test.built_file_path(
      os.path.join(test_app_path, 'Info.plist'),
      chdir='app-bundle')
  # Resources
  test.built_file_must_exist(
      os.path.join(test_app_path, 'English.lproj/InfoPlist.strings'),
      chdir='app-bundle')
  test.built_file_must_exist(
      os.path.join(test_app_path, 'English.lproj/MainMenu.nib'),
      chdir='app-bundle')
  test.built_file_must_exist(
      os.path.join(test_app_path, 'English.lproj/Main_iPhone.storyboardc'),
      chdir='app-bundle')
  test.built_file_must_exist(
      os.path.join(test_app_path, 'Assets.car'),
      chdir='app-bundle')

  # Packaging
  test.built_file_must_exist(
      os.path.join(test_app_path, 'PkgInfo'),
      chdir='app-bundle')
  test.built_file_must_match(
      os.path.join(test_app_path, 'PkgInfo'), 'APPLause',
      chdir='app-bundle')

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Test that the generator output can be written to a different drive on Windows.
"""

import os
import TestGyp
import string
import subprocess
import sys


if sys.platform == 'win32':
  import win32api

  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  def GetFirstFreeDriveLetter():
    """ Returns the first unused Windows drive letter in [A, Z] """
    all_letters = [c for c in string.ascii_uppercase]
    in_use = win32api.GetLogicalDriveStrings()
    free = list(set(all_letters) - set(in_use))
    return free[0]

  output_dir = os.path.join('different-drive', 'output')
  if not os.path.isdir(os.path.abspath(output_dir)):
    os.makedirs(os.path.abspath(output_dir))
  output_drive = GetFirstFreeDriveLetter()
  subprocess.call(['subst', '%c:' % output_drive, os.path.abspath(output_dir)])
  try:
    test.run_gyp('prog.gyp', '--generator-output=%s' % (
        os.path.join(output_drive, 'output')))
    test.build('prog.gyp', test.ALL, chdir=os.path.join(output_drive, 'output'))
    test.built_file_must_exist('program', chdir=os.path.join(output_drive,
                                                             'output'),
                               type=test.EXECUTABLE)
    test.pass_test()
  finally:
    subprocess.call(['subst', '%c:' % output_drive, '/D'])



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Checks that C-only targets aren't linked against libstdc++.
"""

import TestGyp

import re
import subprocess
import sys

# set |match| to ignore build stderr output.
test = TestGyp.TestGyp(match = lambda a, b: True)
if (sys.platform != 'win32' and
    not (sys.platform == 'darwin' and test.format == 'make')):
  # TODO: Does a test like this make sense with Windows?

  CHDIR = 'src'
  test.run_gyp('test.gyp', chdir=CHDIR)
  test.build('test.gyp', 'no_cpp', chdir=CHDIR)

  def LinksLibStdCpp(path):
    path = test.built_file_path(path, chdir=CHDIR)
    if sys.platform == 'darwin':
      proc = subprocess.Popen(['otool', '-L', path], stdout=subprocess.PIPE)
    else:
      proc = subprocess.Popen(['ldd', path], stdout=subprocess.PIPE)
    output = proc.communicate()[0].decode('utf-8')
    assert not proc.returncode
    return 'libstdc++' in output or 'libc++' in output

  if LinksLibStdCpp('no_cpp'):
    test.fail_test()

  # Make, ninja, and CMake pick the compiler driver based on transitive
  # checks. Xcode doesn't.
  build_error_code = {
    'xcode': 65,  # EX_DATAERR, see `man sysexits`
    'make': 0,
    'ninja': 0,
    'cmake': 0,
    'xcode-ninja': 0,
  }[test.format]

  test.build('test.gyp', 'no_cpp_dep_on_cc_lib', chdir=CHDIR,
             status=build_error_code)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Tests things related to ARCHS.
"""

import TestGyp
import TestMac

import re
import subprocess
import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  test.run_gyp('test-no-archs.gyp', chdir='archs')
  test.build('test-no-archs.gyp', test.ALL, chdir='archs')
  result_file = test.built_file_path('Test', chdir='archs')
  test.must_exist(result_file)

  if TestMac.Xcode.Version() >= '0500':
    expected_type = ['x86_64']
  else:
    expected_type = ['i386']
  TestMac.CheckFileType(test, result_file, expected_type)

  test.run_gyp('test-valid-archs.gyp', chdir='archs')
  test.build('test-valid-archs.gyp', test.ALL, chdir='archs')
  result_file = test.built_file_path('Test', chdir='archs')
  test.must_exist(result_file)
  TestMac.CheckFileType(test, result_file, ['x86_64'])

  test.run_gyp('test-archs-x86_64.gyp', chdir='archs')
  test.build('test-archs-x86_64.gyp', test.ALL, chdir='archs')
  result_file = test.built_file_path('Test64', chdir='archs')
  test.must_exist(result_file)
  TestMac.CheckFileType(test, result_file, ['x86_64'])

  test.run_gyp('test-dependencies.gyp', chdir='archs')
  test.build('test-dependencies.gyp', target=test.ALL, chdir='archs')
  products = ['c_standalone', 'd_standalone']
  for product in products:
    result_file = test.built_file_path(
        product, chdir='archs', type=test.STATIC_LIB)
    test.must_exist(result_file)

  if test.format != 'make':
    # Build all targets except 'exe_32_64_no_sources' that does build
    # but should not cause error when generating ninja files
    targets = [
        'static_32_64', 'shared_32_64', 'shared_32_64_bundle',
        'module_32_64', 'module_32_64_bundle',
        'exe_32_64', 'exe_32_64_bundle', 'precompiled_prefix_header_mm_32_64',
    ]

    test.run_gyp('test-archs-multiarch.gyp', chdir='archs')

    for target in targets:
      test.build('test-archs-multiarch.gyp', target=target, chdir='archs')

    result_file = test.built_file_path(
        'static_32_64', chdir='archs', type=test.STATIC_LIB)
    test.must_exist(result_file)
    TestMac.CheckFileType(test, result_file, ['i386', 'x86_64'])

    result_file = test.built_file_path(
        'shared_32_64', chdir='archs', type=test.SHARED_LIB)
    test.must_exist(result_file)
    TestMac.CheckFileType(test, result_file, ['i386', 'x86_64'])

    result_file = test.built_file_path('My Framework.framework/My Framework',
                                       chdir='archs')
    test.must_exist(result_file)
    TestMac.CheckFileType(test, result_file, ['i386', 'x86_64'])
    # Check that symbol "_x" made it into both versions of the binary:
    if not all(['D _x' in subprocess.check_output(
        ['nm', '-arch', arch, result_file]).decode('utf-8')
        for arch in ['i386', 'x86_64']]):
      # This can only flakily fail, due to process ordering issues. If this
      # does fail flakily, then something's broken, it's not the test at fault.
      test.fail_test()

    result_file = test.built_file_path(
        'exe_32_64', chdir='archs', type=test.EXECUTABLE)
    test.must_exist(result_file)
    TestMac.CheckFileType(test, result_file, ['i386', 'x86_64'])

    result_file = test.built_file_path('Test App.app/Contents/MacOS/Test App',
                                       chdir='archs')
    test.must_exist(result_file)
    TestMac.CheckFileType(test, result_file, ['i386', 'x86_64'])



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that xcuitest targets are correctly configured.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['xcode'])

  # Ignore this test if Xcode 5 is not installed
  import subprocess
  job = subprocess.Popen(['xcodebuild', '-version'],
                         stdout=subprocess.PIPE,
                         stderr=subprocess.STDOUT)
  out, err = job.communicate()
  if job.returncode != 0:
    raise Exception('Error %d running xcodebuild' % job.returncode)
  xcode_version, build_number = out.decode('utf-8').splitlines()
  # Convert the version string from 'Xcode 5.0' to ['5','0'].
  xcode_version = xcode_version.split()[-1].split('.')
  if xcode_version < ['7']:
    test.pass_test()

  CHDIR = 'xcuitest'
  test.run_gyp('test.gyp', chdir=CHDIR)
  test.build('test.gyp', chdir=CHDIR, arguments=[
    '-target', 'tests',
    '-sdk', 'iphonesimulator',
  ])

  test.pass_test()



#!/usr/bin/env python

# Copyright 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import sys

f = open(sys.argv[1], 'w')
f.write(' '.join(sys.argv[2:]))
f.close()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure safeseh setting is extracted properly.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp()

  CHDIR = 'linker-flags'
  test.run_gyp('safeseh.gyp', chdir=CHDIR)
  test.build('safeseh.gyp', test.ALL, chdir=CHDIR)

  def HasSafeExceptionHandlers(exe):
    full_path = test.built_file_path(exe, chdir=CHDIR)
    output = test.run_dumpbin('/LOADCONFIG', full_path)
    return '    Safe Exception Handler Table' in output

  # From MSDN: http://msdn.microsoft.com/en-us/library/9a89h429.aspx
  #   If /SAFESEH is not specified, the linker will produce an image with a
  #   table of safe exceptions handlers if all modules are compatible with
  #   the safe exception handling feature. If any modules were not
  #   compatible with safe exception handling feature, the resulting image
  #   will not contain a table of safe exception handlers.
  #   However, the msvs IDE passes /SAFESEH to the linker by default, if
  #   ImageHasSafeExceptionHandlers is not set to false in the vcxproj file.
  #   We emulate this behavior in msvs_emulation.py, so 'test_safeseh_default'
  #   and 'test_safeseh_yes' are built identically.
  if not HasSafeExceptionHandlers('test_safeseh_default.exe'):
    test.fail_test()
  if HasSafeExceptionHandlers('test_safeseh_no.exe'):
    test.fail_test()
  if not HasSafeExceptionHandlers('test_safeseh_yes.exe'):
    test.fail_test()
  if HasSafeExceptionHandlers('test_safeseh_x64.exe'):
    test.fail_test()

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import os
import sys

outfile = sys.argv[1]
open(outfile, 'w').write('const char kFoo[] = "%s";' % sys.argv[2])



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Test variable expansion of '<|(list.txt ...)' syntax commands.
"""

import os
import sys

import TestGyp

test = TestGyp.TestGyp()

CHDIR = 'src'
test.run_gyp('filelist2.gyp', chdir=CHDIR)

test.build('filelist2.gyp', 'foo', chdir=CHDIR)
contents = test.read('src/dummy_foo').replace('\r', '')
expect = 'John\nJacob\nJingleheimer\nSchmidt\n'
if not test.match(contents, expect):
  print("Unexpected contents of `src/dummy_foo'")
  test.diff(expect, contents, 'src/dummy_foo')
  test.fail_test()

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that xcode-style GCC_... settings that require clang are handled
properly.
"""

import TestGyp

import os
import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  CHDIR = 'xcode-gcc'
  test.run_gyp('test-clang.gyp', chdir=CHDIR)

  test.build('test-clang.gyp', 'aliasing_yes', chdir=CHDIR)
  test.run_built_executable('aliasing_yes', chdir=CHDIR, stdout="1\n")
  test.build('test-clang.gyp', 'aliasing_no', chdir=CHDIR)
  test.run_built_executable('aliasing_no', chdir=CHDIR, stdout="0\n")

  # The default behavior changed: strict aliasing used to be off, now it's on
  # by default. The important part is that this is identical for all generators
  # (which it is). TODO(thakis): Enable this once the bots have a newer Xcode.
  #test.build('test-clang.gyp', 'aliasing_default', chdir=CHDIR)
  #test.run_built_executable('aliasing_default', chdir=CHDIR, stdout="1\n")
  # For now, just check the generated ninja file:
  if test.format == 'ninja':
    contents = open(test.built_file_path('obj/aliasing_default.ninja',
                                         chdir=CHDIR)).read()
    if 'strict-aliasing' in contents:
      test.fail_test()

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2009 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies nested dependent_settings directives project generation.
"""

import TestGyp

test = TestGyp.TestGyp()

test.run_gyp("all-dependent-settings.gyp")
test.run_gyp("direct-dependent-settings.gyp")

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure a relink is performed when a .def file is touched.
"""

from __future__ import print_function

import TestGyp

import sys

if sys.platform == 'win32':
  print("This test is currently disabled: https://crbug.com/483696.")
  sys.exit(0)

  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'linker-flags'
  target = 'test_deffile_dll_ok'
  def_contents = test.read('linker-flags/deffile.def')

  # This first build makes sure everything is up to date.
  test.run_gyp('deffile.gyp', chdir=CHDIR)
  test.build('deffile.gyp', target, chdir=CHDIR)
  test.up_to_date('deffile.gyp', target, chdir=CHDIR)

  def HasExport(binary, export):
    full_path = test.built_file_path(binary, chdir=CHDIR)
    output = test.run_dumpbin('/exports', full_path)
    return export in output

  # Verify that only one function is exported.
  if not HasExport('test_deffile_dll_ok.dll', 'AnExportedFunction'):
    test.fail_test()
  if HasExport('test_deffile_dll_ok.dll', 'AnotherExportedFunction'):
    test.fail_test()

  # Add AnotherExportedFunction to the def file, then rebuild.  If it doesn't
  # relink the DLL, then the subsequent check for AnotherExportedFunction will
  # fail.
  new_def_contents = def_contents + "\n    AnotherExportedFunction"
  test.write('linker-flags/deffile.def', new_def_contents)
  test.build('deffile.gyp', target, chdir=CHDIR)
  test.up_to_date('deffile.gyp', target, chdir=CHDIR)

  if not HasExport('test_deffile_dll_ok.dll', 'AnExportedFunction'):
    test.fail_test()
  if not HasExport('test_deffile_dll_ok.dll', 'AnotherExportedFunction'):
    test.fail_test()

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies libraries have proper mtime.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  CHDIR = 'libtool-zero'

  test.run_gyp('test.gyp', chdir=CHDIR)

  test.build('test.gyp', 'mylib', chdir=CHDIR)

  test.up_to_date('test.gyp', 'mylib', chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Ensure that ninja includes the .pdb as an output file from linking.
"""

import TestGyp

import os
import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['ninja'])
  CHDIR = 'linker-flags'
  test.run_gyp('pdb-output.gyp', chdir=CHDIR)
  # Note, building the pdbs rather than ALL or gyp target.
  test.build('pdb-output.gyp', 'output_exe.pdb', chdir=CHDIR)
  test.build('pdb-output.gyp', 'output_dll.pdb', chdir=CHDIR)

  def FindFile(pdb):
    full_path = test.built_file_path(pdb, chdir=CHDIR)
    return os.path.isfile(full_path)

  if not FindFile('output_exe.pdb'):
    test.fail_test()
  if not FindFile('output_dll.pdb'):
    test.fail_test()

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies --root-target removes the unnecessary targets.
"""

import TestGyp

test = TestGyp.TestGyp()
# The xcode-ninja generator has its own logic for which targets to include
if test.format == 'xcode-ninja':
  test.skip_test()

build_error_code = {
  'cmake': 1,
  'make': 2,
  'msvs': 1,
  'ninja': 1,
  'xcode': 65,
}[test.format]

# By default, everything will be included.
test.run_gyp('test1.gyp')
test.build('test2.gyp', 'lib1')
test.build('test2.gyp', 'lib2')
test.build('test2.gyp', 'lib3')
test.build('test2.gyp', 'lib_indirect')
test.build('test1.gyp', 'program1')
test.build('test1.gyp', 'program2')
test.build('test1.gyp', 'program3')

# With deep dependencies of program1 only.
test.run_gyp('test1.gyp', '--root-target=program1')
test.build('test2.gyp', 'lib1')
test.build('test2.gyp', 'lib2', status=build_error_code, stderr=None)
test.build('test2.gyp', 'lib3', status=build_error_code, stderr=None)
test.build('test2.gyp', 'lib_indirect')
test.build('test1.gyp', 'program1')
test.build('test1.gyp', 'program2', status=build_error_code, stderr=None)
test.build('test1.gyp', 'program3', status=build_error_code, stderr=None)

# With deep dependencies of program2 only.
test.run_gyp('test1.gyp', '--root-target=program2')
test.build('test2.gyp', 'lib1', status=build_error_code, stderr=None)
test.build('test2.gyp', 'lib2')
test.build('test2.gyp', 'lib3', status=build_error_code, stderr=None)
test.build('test2.gyp', 'lib_indirect')
test.build('test1.gyp', 'program1', status=build_error_code, stderr=None)
test.build('test1.gyp', 'program2')
test.build('test1.gyp', 'program3', status=build_error_code, stderr=None)

# With deep dependencies of program1 and program2.
test.run_gyp('test1.gyp', '--root-target=program1', '--root-target=program2')
test.build('test2.gyp', 'lib1')
test.build('test2.gyp', 'lib2')
test.build('test2.gyp', 'lib3', status=build_error_code, stderr=None)
test.build('test2.gyp', 'lib_indirect')
test.build('test1.gyp', 'program1')
test.build('test1.gyp', 'program2')
test.build('test1.gyp', 'program3', status=build_error_code, stderr=None)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Tests that loadable_modules don't collide when using the same name with
different file extensions.
"""

import TestGyp

import os
import struct
import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  CHDIR = 'loadable-module-bundle-product-extension'
  test.run_gyp('test.gyp',
               '-G', 'xcode_ninja_target_pattern=^.*$',
               chdir=CHDIR)

  test.build('test.gyp', test.ALL, chdir=CHDIR)

  test.must_exist(test.built_file_path('Collide.foo', chdir=CHDIR))
  test.must_exist(test.built_file_path('Collide.bar', chdir=CHDIR))

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies things related to bundle resources.
"""

from __future__ import print_function

import TestGyp

import os
import stat
import sys

if sys.platform in ('darwin'):
  print("This test is currently disabled: https://crbug.com/483696.")
  sys.exit(0)


def check_attribs(path, expected_exec_bit):
  out_path = test.built_file_path(
      os.path.join('resource.app/Contents/Resources', path), chdir=CHDIR)

  in_stat = os.stat(os.path.join(CHDIR, path))
  out_stat = os.stat(out_path)
  if in_stat.st_mtime == out_stat.st_mtime:
    test.fail_test()
  if out_stat.st_mode & stat.S_IXUSR != expected_exec_bit:
    test.fail_test()


if sys.platform == 'darwin':
  # set |match| to ignore build stderr output.
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  CHDIR = 'bundle-resources'
  test.run_gyp('test.gyp', chdir=CHDIR)

  test.build('test.gyp', test.ALL, chdir=CHDIR)

  test.built_file_must_match('resource.app/Contents/Resources/secret.txt',
                             'abc\n', chdir=CHDIR)
  test.built_file_must_match('source_rule.app/Contents/Resources/secret.txt',
                             'ABC\n', chdir=CHDIR)

  test.built_file_must_match(
      'resource.app/Contents/Resources/executable-file.sh',
      '#!/bin/bash\n'
      '\n'
      'echo echo echo echo cho ho o o\n', chdir=CHDIR)

  check_attribs('executable-file.sh', expected_exec_bit=stat.S_IXUSR)
  check_attribs('secret.txt', expected_exec_bit=0)

  # TODO(thakis): This currently fails with make.
  if test.format != 'make':
    test.built_file_must_match(
        'resource_rule.app/Contents/Resources/secret.txt', 'ABC\n', chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Test that two targets with the same name generates an error.
"""

import os
import sys

import TestGyp
import TestCmd

# TODO(sbc): Remove the use of match_re below, done because scons
# error messages were not consistent with other generators.
# Also remove input.py:generator_wants_absolute_build_file_paths.

test = TestGyp.TestGyp()

stderr = ('gyp: Duplicate target definitions for '
          '.*duplicate_targets.gyp:foo#target\n')
test.run_gyp('duplicate_targets.gyp', status=1, stderr=stderr,
             match=TestCmd.match_re)

stderr = ('.*: Unable to find targets in build file .*missing_targets.gyp.*')
test.run_gyp('missing_targets.gyp', status=1, stderr=stderr,
             match=TestCmd.match_re_dotall)

stderr = ('gyp: rule bar exists in duplicate, target '
          '.*duplicate_rule.gyp:foo#target\n')
test.run_gyp('duplicate_rule.gyp', status=1, stderr=stderr,
             match=TestCmd.match_re)

stderr = ("gyp: Key 'targets' repeated at level 1 with key path '' while "
          "reading .*duplicate_node.gyp.*")
test.run_gyp('duplicate_node.gyp', '--check', status=1, stderr=stderr,
             match=TestCmd.match_re_dotall)

stderr = (".*target0.*target1.*target2.*target0.*")
test.run_gyp('dependency_cycle.gyp', status=1, stderr=stderr,
             match=TestCmd.match_re_dotall)

stderr = (".*file_cycle0.*file_cycle1.*file_cycle0.*")
test.run_gyp('file_cycle0.gyp', status=1, stderr=stderr,
             match=TestCmd.match_re_dotall)

stderr = 'gyp: Duplicate basenames in sources section, see list above\n'
test.run_gyp('duplicate_basenames.gyp', status=1, stderr=stderr)

# Check if '--no-duplicate-basename-check' works.
if ((test.format == 'make' and sys.platform == 'darwin') or
    (test.format == 'msvs' and
        int(os.environ.get('GYP_MSVS_VERSION', 2010)) < 2010)):
  stderr = 'gyp: Duplicate basenames in sources section, see list above\n'
  test.run_gyp('duplicate_basenames.gyp', '--no-duplicate-basename-check',
               status=1, stderr=stderr)
else:
  test.run_gyp('duplicate_basenames.gyp', '--no-duplicate-basename-check')

stderr = ("gyp: Dependency '.*missing_dep.gyp:missing.gyp#target' not found "
          "while trying to load target .*missing_dep.gyp:foo#target\n")
test.run_gyp('missing_dep.gyp', status=1, stderr=stderr,
             match=TestCmd.match_re)

# Make sure invalid <!() command invocations say what command it was and
# mention the gyp file name. Use a "random" command name to trigger an ENOENT.
stderr = (".*invalid-command-name-egtyevNif3.*netDurj9.*missing_command.gyp.*")
test.run_gyp('missing_command.gyp', status=1, stderr=stderr,
             match=TestCmd.match_re_dotall)

# Make sure <!() commands that error out result in a message that mentions
# the command and gyp file name
stderr = (".*python.*-c.*import sys.*sys.exit.*3.*error_command.gyp.*")
test.run_gyp('error_command.gyp', status=1, stderr=stderr,
             match=TestCmd.match_re_dotall)

test.pass_test()



# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import sys

with open('external_builder.out', 'w') as f:
  f.write(' '.join(sys.argv))




#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure the link order of object files is the same between msvs and ninja.
"""

from __future__ import print_function

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'linker-flags'
  test.run_gyp('link-ordering.gyp', chdir=CHDIR)
  test.build('link-ordering.gyp', test.ALL, chdir=CHDIR)

  def GetDisasm(exe):
    full_path = test.built_file_path(exe, chdir=CHDIR)
    # Get disassembly and drop int3 padding between functions.
    return '\n'.join(
        x for x in test.run_dumpbin('/disasm', full_path).splitlines()
                   if 'CC' not in x)

  # This is the full dump that we expect. The source files in the .gyp match
  # this order which is what determines the ordering in the binary.

  expected_disasm_basic = '''
_mainCRTStartup:
  00401000: B8 05 00 00 00     mov         eax,5
  00401005: C3                 ret
?z@@YAHXZ:
  00401010: B8 03 00 00 00     mov         eax,3
  00401015: C3                 ret
?x@@YAHXZ:
  00401020: B8 01 00 00 00     mov         eax,1
  00401025: C3                 ret
?y@@YAHXZ:
  00401030: B8 02 00 00 00     mov         eax,2
  00401035: C3                 ret
_main:
  00401040: 33 C0              xor         eax,eax
  00401042: C3                 ret
'''

  if expected_disasm_basic not in GetDisasm('test_ordering_exe.exe'):
    print(GetDisasm('test_ordering_exe.exe'))
    test.fail_test()

  # Similar to above. The VS generator handles subdirectories differently.

  expected_disasm_subdirs = '''
_mainCRTStartup:
  00401000: B8 05 00 00 00     mov         eax,5
  00401005: C3                 ret
_main:
  00401010: 33 C0              xor         eax,eax
  00401012: C3                 ret
?y@@YAHXZ:
  00401020: B8 02 00 00 00     mov         eax,2
  00401025: C3                 ret
?z@@YAHXZ:
  00401030: B8 03 00 00 00     mov         eax,3
  00401035: C3                 ret
'''

  if expected_disasm_subdirs not in GetDisasm('test_ordering_subdirs.exe'):
    print(GetDisasm('test_ordering_subdirs.exe'))
    test.fail_test()

  # Similar, but with directories mixed into folders (crt and main at the same
  # level, but with a subdir in the middle).

  expected_disasm_subdirs_mixed = '''
_mainCRTStartup:
  00401000: B8 05 00 00 00     mov         eax,5
  00401005: C3                 ret
?x@@YAHXZ:
  00401010: B8 01 00 00 00     mov         eax,1
  00401015: C3                 ret
_main:
  00401020: 33 C0              xor         eax,eax
  00401022: C3                 ret
?z@@YAHXZ:
  00401030: B8 03 00 00 00     mov         eax,3
  00401035: C3                 ret
?y@@YAHXZ:
  00401040: B8 02 00 00 00     mov         eax,2
  00401045: C3                 ret
'''

  if (expected_disasm_subdirs_mixed not in
      GetDisasm('test_ordering_subdirs_mixed.exe')):
    print(GetDisasm('test_ordering_subdirs_mixed.exe'))
    test.fail_test()

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that device and simulator bundles are built correctly.
"""

import TestGyp
import TestMac

import collections
import sys


if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'xcode'])

  if test.format == 'xcode':
    # This test appears to hang flakily.
    test.skip_test()  # bug=532

  test_cases = [
    ('Default', 'TestArch32Bits', ['i386']),
    ('Default-iphoneos', 'TestArch32Bits', ['armv7']),
  ]

  if TestMac.Xcode.Version() < '0510':
    test_cases.extend([
        ('Default', 'TestNoArchs', ['i386']),
        ('Default-iphoneos', 'TestNoArchs', ['armv7'])])

  if TestMac.Xcode.Version() >= '0500':
    test_cases.extend([
        ('Default', 'TestArch64Bits', ['x86_64']),
        ('Default', 'TestMultiArchs', ['i386', 'x86_64']),
        ('Default-iphoneos', 'TestArch64Bits', ['arm64']),
        ('Default-iphoneos', 'TestMultiArchs', ['armv7', 'arm64'])])

  test.run_gyp('test-archs.gyp', chdir='app-bundle')
  for configuration, target, archs in test_cases:
    is_device_build = configuration.endswith('-iphoneos')

    kwds = collections.defaultdict(list)
    if test.format == 'xcode':
      if is_device_build:
        configuration, sdk = configuration.split('-')
        kwds['arguments'].extend(['-sdk', sdk])
      if TestMac.Xcode.Version() < '0500':
        kwds['arguments'].extend(['-arch', archs[0]])

    test.set_configuration(configuration)
    filename = '%s.app/%s' % (target, target)
    test.build('test-archs.gyp', target, chdir='app-bundle', **kwds)
    result_file = test.built_file_path(filename, chdir='app-bundle')

    test.must_exist(result_file)
    TestMac.CheckFileType(test, result_file, archs)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that xctest targets are correctly configured.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['xcode'])

  # This test appears to be flaky.
  test.skip_test()  # bug=531

  # Ignore this test if Xcode 5 is not installed
  import subprocess
  job = subprocess.Popen(['xcodebuild', '-version'],
                         stdout=subprocess.PIPE,
                         stderr=subprocess.STDOUT)
  out, err = job.communicate()
  if job.returncode != 0:
    raise Exception('Error %d running xcodebuild' % job.returncode)
  xcode_version, build_number = out.splitlines()
  # Convert the version string from 'Xcode 5.0' to ['5','0'].
  xcode_version = xcode_version.split()[-1].split('.')
  if xcode_version < ['5']:
    test.pass_test()

  CHDIR = 'xctest'
  test.run_gyp('test.gyp', chdir=CHDIR)
  test.build('test.gyp', chdir=CHDIR, arguments=['-scheme', 'classes', 'test'])

  test.built_file_must_match('tests.xctest/Contents/Resources/resource.txt',
                             'foo\n', chdir=CHDIR)
  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that extra filters are pruned correctly for Visual Studio 2010
and later.
"""

import TestGyp


test = TestGyp.TestGyp(formats=['msvs'])

test.run_gyp('filters.gyp', '-G', 'standalone', '-G', 'msvs_version=2010')

test.must_not_exist('no_source_files.vcxproj.filters')

test.must_not_exist('one_source_file.vcxproj.filters')

test.must_not_exist('two_source_files.vcxproj.filters')

test.must_contain('three_files_in_two_folders.vcxproj.filters', '''\
  <ItemGroup>
    <ClCompile Include="..\\folder1\\a.c">
      <Filter>folder1</Filter>
    </ClCompile>
    <ClCompile Include="..\\folder1\\b.c">
      <Filter>folder1</Filter>
    </ClCompile>
    <ClCompile Include="..\\folder2\\c.c">
      <Filter>folder2</Filter>
    </ClCompile>
  </ItemGroup>
'''.replace('\n', '\r\n'))

test.must_contain('nested_folders.vcxproj.filters', '''\
  <ItemGroup>
    <ClCompile Include="..\\folder1\\nested\\a.c">
      <Filter>folder1\\nested</Filter>
    </ClCompile>
    <ClCompile Include="..\\folder2\\d.c">
      <Filter>folder2</Filter>
    </ClCompile>
    <ClCompile Include="..\\folder1\\nested\\b.c">
      <Filter>folder1\\nested</Filter>
    </ClCompile>
    <ClCompile Include="..\\folder1\\other\\c.c">
      <Filter>folder1\\other</Filter>
    </ClCompile>
  </ItemGroup>
'''.replace('\n', '\r\n'))


test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verify that "else-if" conditions work.
"""

import TestGyp

test = TestGyp.TestGyp()

test.run_gyp('elseif.gyp')
test.build('elseif.gyp', test.ALL)
test.run_built_executable(
    'program0', stdout='first_if\n')
test.run_built_executable(
    'program1', stdout='first_else_if\n')
test.run_built_executable(
    'program2', stdout='second_else_if\n')
test.run_built_executable(
    'program3', stdout='third_else_if\n')
test.run_built_executable(
    'program4', stdout='last_else\n')

# Verify that bad condition blocks fail at gyp time.
test.run_gyp('elseif_bad1.gyp', status=1, stderr=None)
test.run_gyp('elseif_bad2.gyp', status=1, stderr=None)
test.run_gyp('elseif_bad3.gyp', status=1, stderr=None)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure that manual rules on Windows override the built in ones.
"""

import sys
import TestGyp

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])
  CHDIR = 'src'
  test.run_gyp('override.gyp', chdir=CHDIR)
  test.build('override.gyp', test.ALL, chdir=CHDIR)
  expect = """\
Hello from program.c
Got 42.
"""
  test.run_built_executable('program', chdir=CHDIR, stdout=expect)
  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2009 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies simplest-possible build of a "Hello, world!" program
using non-default extension. In particular, verifies how
target_extension is used to avoid MSB8012 for msvs.
"""

import sys
import TestGyp

if sys.platform in ('win32', 'cygwin'):
  test = TestGyp.TestGyp()

  test.run_gyp('target.gyp')
  test.build('target.gyp')

  # executables
  test.built_file_must_exist('hello1.stuff', test.EXECUTABLE, bare=True)
  test.built_file_must_exist('hello2.exe', test.EXECUTABLE, bare=True)
  test.built_file_must_not_exist('hello2.stuff', test.EXECUTABLE, bare=True)

  # check msvs log for errors
  if test.format == "msvs":
    log_file = "obj\\hello1\\hello1.log"
    test.built_file_must_exist(log_file)
    test.built_file_must_not_contain(log_file, "MSB8012")

    log_file = "obj\\hello2\\hello2.log"
    test.built_file_must_exist(log_file)
    test.built_file_must_not_contain(log_file, "MSB8012")

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies building a target when the --generator-output= option is used to put
the build configuration files in a separate directory tree referenced by a
symlink.
"""

import TestGyp
import os
import sys

test = TestGyp.TestGyp()
if not hasattr(os, 'symlink') or sys.platform == 'win32':
  # Python3 on windows has symlink but it doesn't work reliably.
  test.skip_test('Missing or bad os.symlink -- skipping test.\n')

test.writable(test.workpath('src'), False)

test.writable(test.workpath('src/subdir2/deeper/build'), True)

test.subdir(test.workpath('build'))
test.subdir(test.workpath('build/deeper'))
test.symlink('build/deeper', test.workpath('symlink'))

test.writable(test.workpath('build/deeper'), True)
test.run_gyp('deeper.gyp',
             '-Dset_symroot=2',
             '--generator-output=' + test.workpath('symlink'),
             chdir='src/subdir2/deeper')

chdir = 'symlink'
test.build('deeper.gyp', test.ALL, chdir=chdir)

if test.format == 'xcode':
  chdir = 'src/subdir2/deeper'
test.run_built_executable('deeper',
                          chdir=chdir,
                          stdout="Hello from deeper.c\n")
test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure that we cause downstream modules to get built when we depend on the
parent targets.
"""

import TestGyp

test = TestGyp.TestGyp()

CHDIR = 'module-dep'
test.run_gyp('indirect-module-dependency.gyp', chdir=CHDIR)
test.build('indirect-module-dependency.gyp', 'an_exe', chdir=CHDIR)
test.built_file_must_exist(
    test.built_file_basename('a_module', test.LOADABLE_MODULE), chdir=CHDIR)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that a swift framework builds correctly.
"""

from __future__ import print_function

import TestGyp
import TestMac

import collections
import sys
import subprocess

if sys.platform == 'darwin':
  print("This test is currently disabled: https://crbug.com/483696.")
  sys.exit(0)

  test = TestGyp.TestGyp(formats=['xcode'])

  # Ensures that the given symbol is present in the given file, by running nm.
  def CheckHasSymbolName(path, symbol):
    output = subprocess.check_output(['nm', '-j', path])
    idx = output.find(symbol)
    if idx == -1:
      print('Swift: Could not find symobl: %s' % symbol)
      test.fail_test()

  test_cases = []

  # Run this for iOS on XCode 6.0 or greater
  if TestMac.Xcode.Version() >= '0600':
    test_cases.append(('Default', 'iphoneos'))
    test_cases.append(('Default', 'iphonesimulator'))

  # Run it for Mac on XCode 6.1 or greater
  if TestMac.Xcode.Version() >= '0610':
    test_cases.append(('Default', None))

  # Generate the project.
  test.run_gyp('test.gyp', chdir='swift-library')

  # Build and verify for each configuration.
  for configuration, sdk in test_cases:
    kwds = collections.defaultdict(list)
    if test.format == 'xcode':
      if sdk is not None:
        kwds['arguments'].extend(['-sdk', sdk])

    test.set_configuration(configuration)
    test.build('test.gyp', 'SwiftFramework', chdir='swift-library', **kwds)

    filename = 'SwiftFramework.framework/SwiftFramework'
    result_file = test.built_file_path(filename, chdir='swift-library')

    test.must_exist(result_file)

    # Check to make sure that our swift class (GypSwiftTest) is present in the
    # built binary
    CheckHasSymbolName(result_file, "C14SwiftFramework12GypSwiftTest")

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Test that RelativePath(s, d) doesn't return a path starting with '..' when
s is textually below d, but is also a symlink to a file that is not below d.

Returning .. in this case would break the Ninja generator in such a case,
because it computes output directories by concatenating paths, and concat'ing
a path starting with .. can unexpectedly erase other parts of the path. It's
difficult to test this directly since the test harness assumes toplevel_dir is
the root of the repository, but this test should at least verify that the
required behavior doesn't change.
"""

import TestGyp
import os
import sys
import tempfile

if sys.platform != 'win32':
  test = TestGyp.TestGyp()

  # Copy hello.gyp and hello.c to temporary named files, which will then be
  # symlinked back and processed. Note that we don't ask gyp to touch the
  # original files at all; they are only there as source material for the copy.
  # That's why hello.gyp references symlink_hello.c instead of hello.c.
  with tempfile.NamedTemporaryFile(mode='w+') as gyp_file:
    with tempfile.NamedTemporaryFile(mode='w+') as c_file:
      with open('hello.gyp') as orig_gyp_file:
        gyp_file.write(orig_gyp_file.read())
        gyp_file.flush()
      with open('hello.c') as orig_c_file:
        c_file.write(orig_c_file.read())
        c_file.flush()
      # We need to flush the files because we want to read them before closing
      # them, since when they are closed they will be deleted.

      # Don't proceed with the test on a system that doesn't let you read from
      # a still-open temporary file.
      if os.path.getsize(gyp_file.name) == 0:
        raise OSError("Copy to temporary file didn't work.")

      symlink_gyp = test.built_file_path('symlink_hello.gyp')
      symlink_c = test.built_file_path('symlink_hello.c')
      outdir = os.path.dirname(symlink_gyp)

      # Make sure the outdir exists.
      try:
        os.makedirs(outdir)
      except OSError:
        if not os.path.isdir(outdir):
          raise
      os.symlink(gyp_file.name, symlink_gyp)
      os.symlink(c_file.name, symlink_c)

      # Run gyp on the symlinked files.
      test.run_gyp(symlink_gyp, chdir=outdir)
      test.build(symlink_gyp, chdir=outdir)
      test.run_built_executable('symlink_hello', stdout="Hello, world!\n",
                                chdir=outdir)

      test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that ios watch extensions and apps are built correctly.
"""

from __future__ import print_function

import TestGyp
import TestMac

import sys

if sys.platform == 'darwin':
  print("This test is currently disabled: https://crbug.com/483696.")
  sys.exit(0)


if sys.platform == 'darwin' and TestMac.Xcode.Version() >= "0620":
  test = TestGyp.TestGyp(formats=['ninja', 'xcode'])

  test.run_gyp('watch.gyp', chdir='watch')

  test.build(
      'watch.gyp',
      'WatchContainer',
      chdir='watch')

  # Test that the extension exists
  test.built_file_must_exist(
      'WatchContainer.app/PlugIns/WatchKitExtension.appex',
      chdir='watch')

  # Test that the watch app exists
  test.built_file_must_exist(
      'WatchContainer.app/PlugIns/WatchKitExtension.appex/WatchApp.app',
      chdir='watch')

  test.pass_test()




# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import sys, os.path

sys.argv[1] = os.path.basename(sys.argv[1])

with open('msbuild_rule.out', 'w') as f:
  f.write(' '.join(sys.argv))




#!/usr/bin/env python

# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure compile as managed (clr) settings are extracted properly.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp()

  CHDIR = 'compiler-flags'
  test.run_gyp('compile-as-managed.gyp', chdir=CHDIR)
  test.build('compile-as-managed.gyp', "test-compile-as-managed", chdir=CHDIR)
  # Must fail.
  test.build('compile-as-managed.gyp', "test-compile-as-unmanaged",
    chdir=CHDIR, status=1)
  test.pass_test()



#!/usr/bin/env python

# Copyright 2016 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that ios app frameworks are built correctly.
"""

import TestGyp
import TestMac
import subprocess
import sys

if sys.platform == 'darwin' and TestMac.Xcode.Version()>="0700":

  test = TestGyp.TestGyp(formats=['ninja'])
  if test.format == 'xcode-ninja':
    test.skip_test()

  test.run_gyp('framework.gyp', chdir='framework')

  test.build('framework.gyp', 'iOSFramework', chdir='framework')

  test.built_file_must_exist(
      'iOSFramework.framework/Headers/iOSFramework.h',
      chdir='framework')
  test.built_file_must_exist(
      'iOSFramework.framework/Headers/Thing.h',
      chdir='framework')
  test.built_file_must_exist(
      'iOSFramework.framework/iOSFramework',
      chdir='framework')

  test.pass_test()




#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that copying files preserves file attributes.
"""

import TestGyp

import os
import stat
import sys


def check_attribs(path, expected_exec_bit):
  out_path = test.built_file_path(path, chdir='src')

  in_stat = os.stat(os.path.join('src', path))
  out_stat = os.stat(out_path)
  if out_stat.st_mode & stat.S_IXUSR != expected_exec_bit:
    test.fail_test()


test = TestGyp.TestGyp()

test.run_gyp('copies-attribs.gyp', chdir='src')

test.build('copies-attribs.gyp', chdir='src')

if sys.platform != 'win32':
  out_path = test.built_file_path('executable-file.sh', chdir='src')
  test.must_contain(out_path,
                    '#!/bin/bash\n'
                    '\n'
                    'echo echo echo echo cho ho o o\n')
  check_attribs('executable-file.sh', expected_exec_bit=stat.S_IXUSR)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Checks that gyp does not fail on executable targets which have several files
with the same basename.
"""

import TestGyp

# While MSVS supports building executables that contain several files with the
# same name, the msvs gyp generator does not.
test = TestGyp.TestGyp(formats=['!msvs'])

test.run_gyp('double-executable.gyp', chdir='src')

test.relocate('src', 'relocate/src')

test.build('double-executable.gyp', test.ALL, chdir='relocate/src')

expect = """\
Hello from prog3.c
Hello prog3 from func.c
Hello prog3 from subdir1/func.c
Hello prog3 from subdir2/func.c
"""

test.run_built_executable('prog3', chdir='relocate/src', stdout=expect)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure we error out if #pragma comments are used to modify manifests.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  # This assertion only applies to the ninja build.
  test = TestGyp.TestGyp(formats=['ninja'])

  CHDIR = 'linker-flags'
  test.run_gyp('unsupported-manifest.gyp', chdir=CHDIR)

  # Just needs to fail to build.
  test.build('unsupported-manifest.gyp',
      'test_unsupported', chdir=CHDIR, status=1)
  test.must_not_exist(test.built_file_path('test_unsupported.exe', chdir=CHDIR))

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that support actions are properly created.
"""

import TestGyp

import os
import subprocess
import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['xcode'])

  CHDIR = 'xcode-support-actions'

  test.run_gyp('test.gyp', '-Gsupport_target_suffix=_customsuffix', chdir=CHDIR)
  test.build('test.gyp', target='target_customsuffix', chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2011 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that msvs_prebuild and msvs_postbuild can be specified in both
VS 2008 and 2010.
"""

import TestGyp

test = TestGyp.TestGyp(formats=['msvs'], workdir='workarea_all')

test.run_gyp('buildevents.gyp', '-G', 'msvs_version=2008')
test.must_contain('main.vcproj', 'Name="VCPreBuildEventTool"')
test.must_contain('main.vcproj', 'Name="VCPostBuildEventTool"')

test.run_gyp('buildevents.gyp', '-G', 'msvs_version=2010')
test.must_contain('main.vcxproj', '<PreBuildEvent>')
test.must_contain('main.vcxproj', '<PostBuildEvent>')

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2009 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that a default gyp define can be overridden.
"""

import os
import TestGyp

test = TestGyp.TestGyp()

# CMake loudly warns about passing '#' to the compiler and drops the define.
expect_stderr = ''
if test.format == 'cmake':
  expect_stderr = (
"""WARNING: Preprocessor definitions containing '#' may not be passed on the"""
""" compiler command line because many compilers do not support it.\n"""
"""CMake is dropping a preprocessor definition: HASH_VALUE="a#1"\n"""
"""Consider defining the macro in a (configured) header file.\n\n""")

# Command-line define
test.run_gyp('defines.gyp', '-D', 'OS=fakeos')
test.build('defines.gyp', stderr=expect_stderr)
test.built_file_must_exist('fakeosprogram', type=test.EXECUTABLE)
# Clean up the exe so subsequent tests don't find an old exe.
os.remove(test.built_file_path('fakeosprogram', type=test.EXECUTABLE))

# Without "OS" override, fokeosprogram shouldn't be built.
test.run_gyp('defines.gyp')
test.build('defines.gyp', stderr=expect_stderr)
test.built_file_must_not_exist('fakeosprogram', type=test.EXECUTABLE)

# Environment define
os.environ['GYP_DEFINES'] = 'OS=fakeos'
test.run_gyp('defines.gyp')
test.build('defines.gyp', stderr=expect_stderr)
test.built_file_must_exist('fakeosprogram', type=test.EXECUTABLE)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that xcode-style GCC_... settings are handled properly.
"""

import TestGyp

import os
import subprocess
import sys

def IgnoreOutput(string, expected_string):
  return True

def CompilerVersion(compiler):
  stdout = subprocess.check_output([compiler, '-v'], stderr=subprocess.STDOUT)
  stdout = stdout.decode('utf-8')
  return stdout.rstrip('\n')

def CompilerSupportsWarnAboutInvalidOffsetOfMacro(test):
  # "clang" does not support the "-Winvalid-offsetof" flag, and silently
  # ignore it. Starting with Xcode 5.0.0, "gcc" is just a "clang" binary with
  # some hard-coded include path hack, so use the output of "-v" to detect if
  # the compiler supports the flag or not.
  return 'clang' not in CompilerVersion('/usr/bin/cc')

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  if test.format == 'xcode-ninja':
    test.skip_test()

  CHDIR = 'xcode-gcc'
  test.run_gyp('test.gyp', chdir=CHDIR)

  # List of targets that'll pass. It expects targets of the same name with
  # '-fail' appended that'll fail to build.
  targets = [
    'warn_about_missing_newline',
  ]

  # clang doesn't warn on invalid offsetofs, it silently ignores
  # -Wno-invalid-offsetof.
  if CompilerSupportsWarnAboutInvalidOffsetOfMacro(test):
    targets.append('warn_about_invalid_offsetof_macro')

  for target in targets:
    test.build('test.gyp', target, chdir=CHDIR)
    test.built_file_must_exist(target, chdir=CHDIR)
    fail_target = target + '-fail'
    test.build('test.gyp', fail_target, chdir=CHDIR, status=None,
               stderr=None, match=IgnoreOutput)
    test.built_file_must_not_exist(fail_target, chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2011 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies simple rules when using an explicit build target of 'all'.
"""

from __future__ import print_function

import TestGyp
import os
import sys

if sys.platform == 'win32':
  print("This test is currently disabled: https://crbug.com/483696.")
  sys.exit(0)


test = TestGyp.TestGyp(formats=['make', 'ninja', 'xcode', 'msvs'])

test.run_gyp('actions.gyp', chdir='src')

test.relocate('src', 'relocate/src')

test.build('actions.gyp', chdir='relocate/src')

expect = """\
no dir here
hi c
hello baz
"""
if test.format == 'xcode':
  chdir = 'relocate/src/subdir'
else:
  chdir = 'relocate/src'
test.run_built_executable('gencc_int_output', chdir=chdir, stdout=expect)
if test.format == 'msvs':
  test.run_built_executable('gencc_int_output_external', chdir=chdir,
                            stdout=expect)

test.must_match('relocate/src/subdir/foo/bar/baz.dirname',
                os.path.join('foo', 'bar'))
test.must_match('relocate/src/subdir/a/b/c.dirname',
                os.path.join('a', 'b'))

# FIXME the xcode and make generators incorrectly convert RULE_INPUT_PATH
# to an absolute path, making the tests below fail!
if test.format != 'xcode' and test.format != 'make':
  test.must_match('relocate/src/subdir/foo/bar/baz.path',
                  os.path.join('foo', 'bar', 'baz.printvars'))
  test.must_match('relocate/src/subdir/a/b/c.path',
                  os.path.join('a', 'b', 'c.printvars'))

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that LTO flags work.
"""

import TestGyp

import os
import re
import subprocess
import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  CHDIR = 'lto'
  test.run_gyp('test.gyp', chdir=CHDIR)

  test.build('test.gyp', test.ALL, chdir=CHDIR)

  def ObjPath(srcpath, target):
    # TODO: Move this into TestGyp if it's needed elsewhere.
    if test.format == 'xcode':
      return os.path.join(CHDIR, 'build', 'test.build', 'Default',
                          target + '.build', 'Objects-normal', 'x86_64',
                          srcpath + '.o')
    elif 'ninja' in test.format:  # ninja, xcode-ninja
      return os.path.join(CHDIR, 'out', 'Default', 'obj',
                          target + '.' + srcpath + '.o')
    elif test.format == 'make':
      return os.path.join(CHDIR, 'out', 'Default', 'obj.target',
                          target, srcpath + '.o')

  def ObjType(p, t_expected):
    r = re.compile(r'nsyms\s+(\d+)')
    o = subprocess.check_output(['file', p]).decode('utf-8')
    objtype = 'unknown'
    if ': Mach-O ' in o:
      objtype = 'mach-o'
    elif ': LLVM bitcode' in o:
      objtype = 'llvm'
    if objtype != t_expected:
      print('Expected %s, got %s' % (t_expected, objtype))
      test.fail_test()

  ObjType(ObjPath('cfile', 'lto'), 'llvm')
  ObjType(ObjPath('ccfile', 'lto'), 'llvm')
  ObjType(ObjPath('mfile', 'lto'), 'llvm')
  ObjType(ObjPath('mmfile', 'lto'), 'llvm')
  ObjType(ObjPath('asmfile', 'lto'), 'mach-o')

  ObjType(ObjPath('cfile', 'lto_static'), 'llvm')
  ObjType(ObjPath('ccfile', 'lto_static'), 'llvm')
  ObjType(ObjPath('mfile', 'lto_static'), 'llvm')
  ObjType(ObjPath('mmfile', 'lto_static'), 'llvm')
  ObjType(ObjPath('asmfile', 'lto_static'), 'mach-o')

  test.pass_test()

  # TODO: Probably test for -object_path_lto too, else dsymutil won't be
  # useful maybe?



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies libraries (in identical-names) are properly handeled by xcode.

The names for all libraries participating in this build are:
libtestlib.a - identical-name/testlib
libtestlib.a - identical-name/proxy/testlib
libproxy.a   - identical-name/proxy
The first two libs produce a hash collision in Xcode when Gyp is executed,
because they have the same name and would be copied to the same directory with
Xcode default settings.
For this scenario to work one needs to change the Xcode variables SYMROOT and
CONFIGURATION_BUILD_DIR. Setting these to per-lib-unique directories, avoids
copying the libs into the same directory.

The test consists of two steps. The first one verifies that by setting both
vars, there is no hash collision anymore during Gyp execution and that the libs
can actually be be built. The second one verifies that there is still a hash
collision if the vars are not set and thus the current behavior is preserved.
"""

import TestGyp

import sys

def IgnoreOutput(string, expected_string):
  return True

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['xcode'])


  test.run_gyp('test.gyp', chdir='identical-name')
  test.build('test.gyp', test.ALL, chdir='identical-name')

  test.run_gyp('test-should-fail.gyp', chdir='identical-name')
  test.built_file_must_not_exist('test-should-fail.xcodeproj')

  test.pass_test()




#!/usr/bin/env python

# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies duplicate ldflags are not removed.
"""

import TestGyp

import sys

if sys.platform.startswith('linux'):
  test = TestGyp.TestGyp()

  CHDIR = 'ldflags-duplicates'
  test.run_gyp('test.gyp', chdir=CHDIR)
  test.build('test.gyp', chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that ninja errors out when encountering msvs_prebuild/msvs_postbuild.
"""

import sys
import TestCmd
import TestGyp


if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['ninja'])

  test.run_gyp('buildevents.gyp',
      status=1,
      stderr=r'.*msvs_prebuild not supported \(target main\).*',
      match=TestCmd.match_re_dotall)

  test.run_gyp('buildevents.gyp',
      status=1,
      stderr=r'.*msvs_postbuild not supported \(target main\).*',
      match=TestCmd.match_re_dotall)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Tests that a loadable_module target is built correctly.
"""

from __future__ import print_function

import TestGyp

import os
import struct
import sys

if sys.platform == 'darwin':
  print("This test is currently disabled: https://crbug.com/483696.")
  sys.exit(0)

  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  CHDIR = 'loadable-module'
  test.run_gyp('test.gyp', chdir=CHDIR)

  test.build('test.gyp', test.ALL, chdir=CHDIR)

  # Binary.
  binary = test.built_file_path(
      'test_loadable_module.plugin/Contents/MacOS/test_loadable_module',
      chdir=CHDIR)
  test.must_exist(binary)
  MH_BUNDLE = 8
  if struct.unpack('4I', open(binary, 'rb').read(16))[3] != MH_BUNDLE:
    test.fail_test()

  # Info.plist.
  info_plist = test.built_file_path(
      'test_loadable_module.plugin/Contents/Info.plist', chdir=CHDIR)
  test.must_exist(info_plist)
  test.must_contain(info_plist, """
	<key>CFBundleExecutable</key>
	<string>test_loadable_module</string>
""")

  # PkgInfo.
  test.built_file_must_not_exist(
      'test_loadable_module.plugin/Contents/PkgInfo', chdir=CHDIR)
  test.built_file_must_not_exist(
      'test_loadable_module.plugin/Contents/Resources', chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verify that a target marked as 'link_dependency==1' isn't being pulled into
the 'none' target's dependency (which would otherwise lead to a dependency
cycle in ninja).
"""

import TestGyp

# See https://codereview.chromium.org/177043010/#msg15 for why this doesn't
# work with cmake.
test = TestGyp.TestGyp(formats=['!cmake'])

test.run_gyp('test.gyp')
test.build('test.gyp', 'main')

# If running gyp worked, all is well.
test.pass_test()



#!/usr/bin/env python

# Copyright 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that embedding UAC information into the manifest works.
"""

import TestGyp

import sys
from xml.dom.minidom import parseString

if sys.platform == 'win32':
  import pywintypes
  import win32api
  import winerror

  RT_MANIFEST = 24

  class LoadLibrary(object):
    """Context manager for loading and releasing binaries in Windows.
    Yields the handle of the binary loaded."""
    def __init__(self, path):
      self._path = path
      self._handle = None

    def __enter__(self):
      self._handle = win32api.LoadLibrary(self._path)
      return self._handle

    def __exit__(self, type, value, traceback):
      win32api.FreeLibrary(self._handle)


  def extract_manifest(path, resource_name):
    """Reads manifest from |path| and returns it as a string.
    Returns None is there is no such manifest."""
    with LoadLibrary(path) as handle:
      try:
        return win32api.LoadResource(handle, RT_MANIFEST, resource_name)
      except pywintypes.error as error:
        if error.args[0] == winerror.ERROR_RESOURCE_DATA_NOT_FOUND:
          return None
        else:
          raise

  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])
  CHDIR = 'linker-flags'
  test.run_gyp('enable-uac.gyp', chdir=CHDIR)
  test.build('enable-uac.gyp', test.ALL, chdir=CHDIR)

  # The following binaries must contain a manifest embedded.
  test.fail_test(not extract_manifest(test.built_file_path(
    'enable_uac.exe', chdir=CHDIR), 1))
  test.fail_test(not extract_manifest(test.built_file_path(
    'enable_uac_no.exe', chdir=CHDIR), 1))
  test.fail_test(not extract_manifest(test.built_file_path(
    'enable_uac_admin.exe', chdir=CHDIR), 1))

  # Verify that <requestedExecutionLevel level="asInvoker" uiAccess="false" />
  # is present.
  manifest = parseString(extract_manifest(
      test.built_file_path('enable_uac.exe', chdir=CHDIR), 1))
  execution_level = manifest.getElementsByTagName('requestedExecutionLevel')
  test.fail_test(len(execution_level) != 1)
  execution_level = execution_level[0].attributes

  def _has_key(node, key):
    # 'in' doesn't work with the NamedNodeMap interface in Python2,
    # but 'has_key' was removed from it in Python3, so we need to
    # shim things :(.
    if hasattr(node, 'has_key'):
      return node.has_key(key)
    return key in node

  test.fail_test(not (
      _has_key(execution_level, 'level') and
      _has_key(execution_level, 'uiAccess') and
      execution_level['level'].nodeValue == 'asInvoker' and
      execution_level['uiAccess'].nodeValue == 'false'))

  # Verify that <requestedExecutionLevel> is not in the menifest.
  manifest = parseString(extract_manifest(
      test.built_file_path('enable_uac_no.exe', chdir=CHDIR), 1))
  execution_level = manifest.getElementsByTagName('requestedExecutionLevel')
  test.fail_test(len(execution_level) != 0)

  # Verify that <requestedExecutionLevel level="requireAdministrator"
  # uiAccess="true" /> is present.
  manifest = parseString(extract_manifest(
      test.built_file_path('enable_uac_admin.exe', chdir=CHDIR), 1))
  execution_level = manifest.getElementsByTagName('requestedExecutionLevel')
  test.fail_test(len(execution_level) != 1)
  execution_level = execution_level[0].attributes
  test.fail_test(not (
      _has_key(execution_level, 'level') and
      _has_key(execution_level, 'uiAccess') and
      execution_level['level'].nodeValue == 'requireAdministrator' and
      execution_level['uiAccess'].nodeValue == 'true'))

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure we generate a manifest file when linking binaries, including
handling AdditionalManifestFiles.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  import pywintypes
  import win32api
  import winerror

  RT_MANIFEST = 24

  class LoadLibrary(object):
    """Context manager for loading and releasing binaries in Windows.
    Yields the handle of the binary loaded."""
    def __init__(self, path):
      self._path = path
      self._handle = None

    def __enter__(self):
      self._handle = win32api.LoadLibrary(self._path)
      return self._handle

    def __exit__(self, type, value, traceback):
      win32api.FreeLibrary(self._handle)

  def extract_manifest(path, resource_name):
    """Reads manifest from |path| and returns it as a string.
    Returns None is there is no such manifest."""
    with LoadLibrary(path) as handle:
      try:
        return win32api.LoadResource(handle, RT_MANIFEST, resource_name)
      except pywintypes.error as error:
        if error.args[0] == winerror.ERROR_RESOURCE_DATA_NOT_FOUND:
          return None
        else:
          raise

  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'linker-flags'
  test.run_gyp('generate-manifest.gyp', chdir=CHDIR)
  test.build('generate-manifest.gyp', test.ALL, chdir=CHDIR)

  # Make sure that generation of .generated.manifest does not cause a relink.
  test.run_gyp('generate-manifest.gyp', chdir=CHDIR)
  test.up_to_date('generate-manifest.gyp', test.ALL, chdir=CHDIR)

  def test_manifest(filename, generate_manifest, embedded_manifest,
                    extra_manifest):
    exe_file = test.built_file_path(filename, chdir=CHDIR)
    if not generate_manifest:
      test.must_not_exist(exe_file + '.manifest')
      manifest = extract_manifest(exe_file, 1)
      test.fail_test(manifest)
      return
    if embedded_manifest:
      manifest = extract_manifest(exe_file, 1)
      test.fail_test(not manifest)
    else:
      test.must_exist(exe_file + '.manifest')
      manifest = test.read(exe_file + '.manifest')
      test.fail_test(not manifest)
      test.fail_test(extract_manifest(exe_file, 1))
    if generate_manifest:
      test.must_contain_any_line(manifest, 'requestedExecutionLevel')
    if extra_manifest:
      test.must_contain_any_line(manifest,
                                 '35138b9a-5d96-4fbd-8e2d-a2440225f93a')
      test.must_contain_any_line(manifest,
                                 'e2011457-1546-43c5-a5fe-008deee3d3f0')

  test_manifest('test_generate_manifest_true.exe',
                generate_manifest=True,
                embedded_manifest=False,
                extra_manifest=False)
  test_manifest('test_generate_manifest_false.exe',
                generate_manifest=False,
                embedded_manifest=False,
                extra_manifest=False)
  test_manifest('test_generate_manifest_default.exe',
                generate_manifest=True,
                embedded_manifest=False,
                extra_manifest=False)
  test_manifest('test_generate_manifest_true_as_embedded.exe',
                generate_manifest=True,
                embedded_manifest=True,
                extra_manifest=False)
  test_manifest('test_generate_manifest_false_as_embedded.exe',
                generate_manifest=False,
                embedded_manifest=True,
                extra_manifest=False)
  test_manifest('test_generate_manifest_default_as_embedded.exe',
                generate_manifest=True,
                embedded_manifest=True,
                extra_manifest=False)
  test_manifest('test_generate_manifest_true_with_extra_manifest.exe',
                generate_manifest=True,
                embedded_manifest=False,
                extra_manifest=True)
  test_manifest('test_generate_manifest_false_with_extra_manifest.exe',
                generate_manifest=False,
                embedded_manifest=False,
                extra_manifest=True)
  test_manifest('test_generate_manifest_true_with_extra_manifest_list.exe',
                generate_manifest=True,
                embedded_manifest=False,
                extra_manifest=True)
  test_manifest('test_generate_manifest_false_with_extra_manifest_list.exe',
                generate_manifest=False,
                embedded_manifest=False,
                extra_manifest=True)
  test_manifest('test_generate_manifest_default_embed_default.exe',
                generate_manifest=True,
                embedded_manifest=True,
                extra_manifest=False)
  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that dependencies on generated headers work, even if the header has
a mixed-case file name.
"""

import TestGyp

test = TestGyp.TestGyp()

CHDIR = 'generated-header'

test.run_gyp('test.gyp', chdir=CHDIR)
test.build('test.gyp', 'program', chdir=CHDIR)
test.up_to_date('test.gyp', 'program', chdir=CHDIR)

expect = 'foobar output\n'
test.run_built_executable('program', chdir=CHDIR, stdout=expect)

# Change what's written to the generated header, regyp and rebuild, and check
# that the change makes it to the executable and that the build is clean.
test.sleep()
test.write('generated-header/test.gyp',
           test.read('generated-header/test.gyp').replace('foobar', 'barbaz'))

test.run_gyp('test.gyp', chdir=CHDIR)
test.build('test.gyp', 'program', chdir=CHDIR)
test.up_to_date('test.gyp', 'program', chdir=CHDIR)

expect = 'barbaz output\n'
test.run_built_executable('program', chdir=CHDIR, stdout=expect)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Ensure that when debug information is not output, a pdb is not expected.
"""

import TestGyp

import os
import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp()
  CHDIR = 'linker-flags'
  test.run_gyp('pdb-output.gyp', chdir=CHDIR)
  test.build('pdb-output.gyp', 'test_pdb_output_disabled', chdir=CHDIR)
  # Make sure that the build doesn't expect a PDB to be generated when there
  # will be none.
  test.up_to_date('pdb-output.gyp', 'test_pdb_output_disabled', chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that ios app bundles are built correctly.
"""

import TestGyp

import subprocess
import sys

def CheckFileXMLPropertyList(file):
  output = subprocess.check_output(['file', file])
  if not 'XML 1.0 document text' in output:
    print('File: Expected XML 1.0 document text, got %s' % output)
    test.fail_test()

def CheckFileBinaryPropertyList(file):
  output = subprocess.check_output(['file', file])
  if not 'Apple binary property list' in output:
    print('File: Expected Apple binary property list, got %s' % output)
    test.fail_test()

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['xcode', 'ninja'])

  test.run_gyp('test.gyp', chdir='app-bundle')

  test.build('test.gyp', test.ALL, chdir='app-bundle')

  # Test that the extension is .bundle
  test.built_file_must_exist('Test App Gyp.app/Test App Gyp',
                             chdir='app-bundle')

  # Info.plist
  info_plist = test.built_file_path('Test App Gyp.app/Info.plist',
                                    chdir='app-bundle')
  test.built_file_must_exist(info_plist)
  CheckFileBinaryPropertyList(info_plist)

  # XML Info.plist
  info_plist = test.built_file_path('Test App Gyp XML.app/Info.plist',
                                    chdir='app-bundle')
  CheckFileXMLPropertyList(info_plist)

  # Resources
  strings_file = test.built_file_path(
      'Test App Gyp.app/English.lproj/InfoPlist.strings',
      chdir='app-bundle')
  test.built_file_must_exist(strings_file)
  CheckFileBinaryPropertyList(strings_file)

  extra_plist_file = test.built_file_path(
      'Test App Gyp.app/English.lproj/LanguageMap.plist',
      chdir='app-bundle')
  test.built_file_must_exist(extra_plist_file)
  CheckFileBinaryPropertyList(extra_plist_file)

  test.built_file_must_exist(
      'Test App Gyp.app/English.lproj/MainMenu.nib',
      chdir='app-bundle')
  test.built_file_must_exist(
      'Test App Gyp.app/English.lproj/Main_iPhone.storyboardc',
      chdir='app-bundle')

  # Packaging
  test.built_file_must_exist('Test App Gyp.app/PkgInfo',
                             chdir='app-bundle')
  test.built_file_must_match('Test App Gyp.app/PkgInfo', 'APPLause',
                             chdir='app-bundle')

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that extra filters are pruned correctly for Visual Studio 2008.
"""

import re
import TestGyp


def strip_ws(str):
    return re.sub('^ +', '', str, flags=re.M).replace('\n', '')


test = TestGyp.TestGyp(formats=['msvs'])

test.run_gyp('filters.gyp', '-G', 'standalone', '-G', 'msvs_version=2008')

test.must_contain('no_source_files.vcproj', '<Files/>')

test.must_contain('one_source_file.vcproj', strip_ws('''\
<Files>
  <File RelativePath="..\\folder\\a.c"/>
</Files>
'''))

test.must_contain('two_source_files.vcproj', strip_ws('''\
<Files>
  <File RelativePath="..\\folder\\a.c"/>
  <File RelativePath="..\\folder\\b.c"/>
</Files>
'''))

test.must_contain('three_files_in_two_folders.vcproj', strip_ws('''\
<Files>
  <Filter Name="folder1">
    <File RelativePath="..\\folder1\\a.c"/>
    <File RelativePath="..\\folder1\\b.c"/>
  </Filter>
  <Filter Name="folder2">
    <File RelativePath="..\\folder2\\c.c"/>
  </Filter>
</Files>
'''))

test.must_contain('nested_folders.vcproj', strip_ws('''\
<Files>
  <Filter Name="folder1">
    <Filter Name="nested">
      <File RelativePath="..\\folder1\\nested\\a.c"/>
      <File RelativePath="..\\folder1\\nested\\b.c"/>
    </Filter>
    <Filter Name="other">
      <File RelativePath="..\\folder1\\other\\c.c"/>
    </Filter>
  </Filter>
  <Filter Name="folder2">
    <File RelativePath="..\\folder2\\d.c"/>
  </Filter>
</Files>
'''))


test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that MACOSX_DEPLOYMENT_TARGET works.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['make', 'ninja', 'xcode'])

  if test.format == 'make':
    # This is failing because of a deprecation warning for libstdc++.
    test.skip_test()  # bug=533

  test.run_gyp('deployment-target.gyp', chdir='deployment-target')

  test.build('deployment-target.gyp', test.ALL, chdir='deployment-target')

  test.pass_test()




#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that building a target with invalid arflags fails.
"""

from __future__ import print_function

import os
import sys
import TestGyp

if sys.platform == 'darwin':
  print("This test is currently disabled: https://crbug.com/483696.")
  sys.exit(0)


test = TestGyp.TestGyp(formats=['ninja'])
test.run_gyp('test.gyp')
expected_status = 0 if sys.platform in ['darwin', 'win32'] else 1
test.build('test.gyp', target='lib', status=expected_status)
test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure msvs_enable_winrt works correctly.
"""

from __future__ import print_function

import TestGyp

import os
import sys
import struct

CHDIR = 'enable-winrt'

print('This test is not currently working on the bots: https://code.google.com/p/gyp/issues/detail?id=466')
sys.exit(0)

if (sys.platform == 'win32' and
    int(os.environ.get('GYP_MSVS_VERSION', 0)) >= 2013):
  test = TestGyp.TestGyp(formats=['msvs'])

  test.run_gyp('enable-winrt.gyp', chdir=CHDIR)

  test.build('enable-winrt.gyp', 'enable_winrt_dll', chdir=CHDIR)

  test.build('enable-winrt.gyp', 'enable_winrt_missing_dll', chdir=CHDIR,
             status=1)

  test.build('enable-winrt.gyp', 'enable_winrt_winphone_dll', chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies 'AR' in make_global_settings.
"""

import os
import sys
import TestGyp

def resolve_path(test, path):
  if path is None:
    return None
  elif test.format == 'make':
    return '$(abspath %s)' % path
  elif test.format in ['ninja', 'xcode-ninja']:
    return os.path.join('..', '..', path)
  else:
    test.fail_test()


def verify_ar_target(test, ar=None, rel_path=False):
  if rel_path:
    ar_expected = resolve_path(test, ar)
  else:
    ar_expected = ar
  # Resolve default values
  if ar_expected is None:
    if test.format == 'make':
      # Make generator hasn't set the default value for AR.
      # You can remove the following assertion as long as it doesn't
      # break existing projects.
      test.must_not_contain('Makefile', 'AR ?= ')
      return
    elif test.format in ['ninja', 'xcode-ninja']:
      if sys.platform == 'win32':
        ar_expected = 'lib.exe'
      else:
        ar_expected = 'ar'
  if test.format == 'make':
    test.must_contain('Makefile', 'AR ?= %s' % ar_expected)
  elif test.format in ['ninja', 'xcode-ninja']:
    test.must_contain('out/Default/build.ninja', 'ar = %s' % ar_expected)
  else:
    test.fail_test()


def verify_ar_host(test, ar=None, rel_path=False):
  if rel_path:
    ar_expected = resolve_path(test, ar)
  else:
    ar_expected = ar
  # Resolve default values
  if ar_expected is None:
    if sys.platform == 'win32':
      ar_expected = 'lib.exe'
    else:
      ar_expected = 'ar'
  if test.format == 'make':
    test.must_contain('Makefile', 'AR.host ?= %s' % ar_expected)
  elif test.format in ['ninja', 'xcode-ninja']:
    test.must_contain('out/Default/build.ninja', 'ar_host = %s' % ar_expected)
  else:
    test.fail_test()


test_format = ['ninja']
if sys.platform.startswith('linux') or sys.platform == 'darwin':
  test_format += ['make']

test = TestGyp.TestGyp(formats=test_format)

# Check default values
test.run_gyp('make_global_settings_ar.gyp')
verify_ar_target(test)


# Check default values with GYP_CROSSCOMPILE enabled.
with TestGyp.LocalEnv({'GYP_CROSSCOMPILE': '1'}):
  test.run_gyp('make_global_settings_ar.gyp')
verify_ar_target(test)
verify_ar_host(test)


# Test 'AR' in 'make_global_settings'.
with TestGyp.LocalEnv({'GYP_CROSSCOMPILE': '1'}):
  test.run_gyp('make_global_settings_ar.gyp', '-Dcustom_ar_target=my_ar')
verify_ar_target(test, ar='my_ar', rel_path=True)


# Test 'AR'/'AR.host' in 'make_global_settings'.
with TestGyp.LocalEnv({'GYP_CROSSCOMPILE': '1'}):
  test.run_gyp('make_global_settings_ar.gyp',
               '-Dcustom_ar_target=my_ar_target1',
               '-Dcustom_ar_host=my_ar_host1')
verify_ar_target(test, ar='my_ar_target1', rel_path=True)
verify_ar_host(test, ar='my_ar_host1', rel_path=True)


# Test $AR and $AR_host environment variables.
with TestGyp.LocalEnv({'AR': 'my_ar_target2',
                       'AR_host': 'my_ar_host2'}):
  test.run_gyp('make_global_settings_ar.gyp')
# Ninja generator resolves $AR in gyp phase. Make generator doesn't.
if test.format == 'ninja':
  if sys.platform == 'win32':
    # TODO(yukawa): Make sure if this is an expected result or not.
    verify_ar_target(test, ar='lib.exe', rel_path=False)
  else:
    verify_ar_target(test, ar='my_ar_target2', rel_path=False)
verify_ar_host(test, ar='my_ar_host2', rel_path=False)


# Test 'AR' in 'make_global_settings' with $AR_host environment variable.
with TestGyp.LocalEnv({'AR_host': 'my_ar_host3'}):
  test.run_gyp('make_global_settings_ar.gyp',
               '-Dcustom_ar_target=my_ar_target3')
verify_ar_target(test, ar='my_ar_target3', rel_path=True)
verify_ar_host(test, ar='my_ar_host3', rel_path=False)


test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure mapfile settings are extracted properly.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'linker-flags'
  test.run_gyp('mapfile.gyp', chdir=CHDIR)
  test.build('mapfile.gyp', test.ALL, chdir=CHDIR)

  map_file = test.built_file_path('test_mapfile_unset.map', chdir=CHDIR)
  test.must_not_exist(map_file)

  map_file = test.built_file_path('test_mapfile_generate.map', chdir=CHDIR)
  test.must_exist(map_file)
  test.must_contain(map_file, '?AnExportedFunction@@YAXXZ')
  test.must_not_contain(map_file, 'void __cdecl AnExportedFunction(void)')

  map_file = test.built_file_path('test_mapfile_generate_exports.map',
          chdir=CHDIR)
  test.must_exist(map_file)
  test.must_contain(map_file, 'void __cdecl AnExportedFunction(void)')

  map_file = test.built_file_path('test_mapfile_generate_filename.map',
          chdir=CHDIR)
  test.must_not_exist(map_file)

  map_file = test.built_file_path('custom_file_name.map', chdir=CHDIR)
  test.must_exist(map_file)
  test.must_contain(map_file, '?AnExportedFunction@@YAXXZ')
  test.must_not_contain(map_file, 'void __cdecl AnExportedFunction(void)')

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies mac bundles work with --generator-output.
"""

from __future__ import print_function

import TestGyp

import sys

if sys.platform == 'darwin':
  print("This test is currently disabled: https://crbug.com/483696.")
  sys.exit(0)

  test = TestGyp.TestGyp(formats=[])

  MAC_BUNDLE_DIR = 'mac-bundle'
  GYPFILES_DIR = 'gypfiles'
  test.writable(test.workpath(MAC_BUNDLE_DIR), False)
  test.run_gyp('test.gyp',
               '--generator-output=' + test.workpath(GYPFILES_DIR),
               chdir=MAC_BUNDLE_DIR)
  test.writable(test.workpath(MAC_BUNDLE_DIR), True)

  test.build('test.gyp', test.ALL, chdir=GYPFILES_DIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure we don't cause unnecessary builds due to import libs appearing
to be out of date.
"""

from __future__ import print_function

import TestGyp

import os
import sys
import time

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  if not os.environ.get('ProgramFiles(x86)'):
    # TODO(scottmg)
    print('Skipping test on x86, http://crbug.com/365833')
    test.pass_test()

  CHDIR = 'importlib'
  test.run_gyp('importlib.gyp', chdir=CHDIR)
  test.build('importlib.gyp', test.ALL, chdir=CHDIR)

  # Delay briefly so that there's time for this touch not to have the
  # timestamp as the previous run.
  test.sleep()

  # Touch the .cc file; the .dll will rebuild, but the import libs timestamp
  # won't be updated.
  test.touch('importlib/has-exports.cc')
  test.build('importlib.gyp', 'test_importlib', chdir=CHDIR)

  # This is the important part. The .dll above will relink and have an updated
  # timestamp, however the import .libs timestamp won't be updated. So, we
  # have to handle restating inputs in ninja so the final binary doesn't
  # continually relink (due to thinking the .lib isn't up to date).
  test.up_to_date('importlib.gyp', test.ALL, chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that unicode strings in 'xcode_settings' work.
Also checks that ASCII control characters are escaped properly.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['xcode'])
  test.run_gyp('test.gyp', chdir='unicode-settings')
  test.build('test.gyp', test.ALL, chdir='unicode-settings')
  test.pass_test()



#!/usr/bin/env python
# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.
"""
Verifies that make_global_settings can be used to override the
compiler settings.
"""
from __future__ import print_function

import TestGyp
import os
import copy
import sys
from string import Template


if sys.platform == 'win32':
  # cross compiling not support by ninja on windows
  # and make not supported on windows at all.
  sys.exit(0)

print("This test is currently disabled: https://crbug.com/483696.")
sys.exit(0)

test = TestGyp.TestGyp(formats=['ninja', 'make'])

gypfile = 'compiler-global-settings.gyp'

replacements = { 'PYTHON': '/usr/bin/python', 'PWD': os.getcwd()}

# Process the .in gyp file to produce the final gyp file
# since we need to include absolute paths in the make_global_settings
# section.
replacements['TOOLSET'] = 'target'
s = Template(open(gypfile + '.in').read())
output = open(gypfile, 'w')
output.write(s.substitute(replacements))
output.close()

old_env = dict(os.environ)
os.environ['GYP_CROSSCOMPILE'] = '1'
test.run_gyp(gypfile)
os.environ.clear()
os.environ.update(old_env)

test.build(gypfile)
test.must_contain_all_lines(test.stdout(), ['my_cc.py', 'my_cxx.py', 'FOO'])

# The xcode generator chokes on the 'host' toolset. Skip the rest of
# this test (cf. https://code.google.com/p/gyp/issues/detail?id=454).
if test.format == 'xcode-ninja':
  test.pass_test()

# Same again but with the host toolset.
replacements['TOOLSET'] = 'host'
s = Template(open(gypfile + '.in').read())
output = open(gypfile, 'w')
output.write(s.substitute(replacements))
output.close()

old_env = dict(os.environ)
os.environ['GYP_CROSSCOMPILE'] = '1'
test.run_gyp(gypfile)
os.environ.clear()
os.environ.update(old_env)

test.build(gypfile)
test.must_contain_all_lines(test.stdout(), ['my_cc.py', 'my_cxx.py', 'BAR'])

# Check that CC_host overrides make_global_settings
old_env = dict(os.environ)
os.environ['CC_host'] = '%s %s/my_cc.py SECRET' % (replacements['PYTHON'],
                                                   replacements['PWD'])
test.run_gyp(gypfile)
os.environ.clear()
os.environ.update(old_env)

test.build(gypfile)
test.must_contain_all_lines(test.stdout(), ['SECRET', 'my_cxx.py', 'BAR'])

test.pass_test()



#!/usr/bin/env python

# Copyright 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure the base address setting is extracted properly.
"""

import TestGyp

import re
import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'linker-flags'
  test.run_gyp('base-address.gyp', chdir=CHDIR)
  test.build('base-address.gyp', test.ALL, chdir=CHDIR)

  def GetHeaders(exe):
    full_path = test.built_file_path(exe, chdir=CHDIR)
    return test.run_dumpbin('/headers', full_path)

  # Extract the image base address from the headers output.
  image_base_reg_ex = re.compile(r'.*\s+([0-9]+) image base.*', re.DOTALL)

  exe_headers = GetHeaders('test_base_specified_exe.exe')
  exe_match = image_base_reg_ex.match(exe_headers)

  if not exe_match or not exe_match.group(1):
    test.fail_test()
  if exe_match.group(1) != '420000':
    test.fail_test()

  dll_headers = GetHeaders('test_base_specified_dll.dll')
  dll_match = image_base_reg_ex.match(dll_headers)

  if not dll_match or not dll_match.group(1):
    test.fail_test()
  if dll_match.group(1) != '10420000':
    test.fail_test()

  default_exe_headers = GetHeaders('test_base_default_exe.exe')
  default_exe_match = image_base_reg_ex.match(default_exe_headers)

  if not default_exe_match or not default_exe_match.group(1):
    test.fail_test()
  if default_exe_match.group(1) != '400000':
    test.fail_test()

  default_dll_headers = GetHeaders('test_base_default_dll.dll')
  default_dll_match = image_base_reg_ex.match(default_dll_headers)

  if not default_dll_match or not default_dll_match.group(1):
    test.fail_test()
  if default_dll_match.group(1) != '10000000':
    test.fail_test()

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2011 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies build of an executable with C++ defines.
"""

import TestGyp

test = TestGyp.TestGyp()

test.run_gyp('defines.gyp')

expect = """\
FOO is defined
VALUE is 1
2*PAREN_VALUE is 12
"""

#CMake loudly warns about passing '#' to the compiler and drops the define.
expect_stderr = ''
if test.format == 'cmake':
  expect_stderr = (
"""WARNING: Preprocessor definitions containing '#' may not be passed on the"""
""" compiler command line because many compilers do not support it.\n"""
"""CMake is dropping a preprocessor definition: HASH_VALUE="a#1"\n"""
"""Consider defining the macro in a (configured) header file.\n\n""")
else:
  expect += """HASH_VALUE is a#1
"""

test.build('defines.gyp', stderr=expect_stderr)

test.run_built_executable('defines', stdout=expect)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2014 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verify that running gyp in a different directory does not cause actions and
rules to rerun.
"""

import os
import sys
import TestGyp

test = TestGyp.TestGyp(formats=['ninja'])
# The xcode-ninja generator handles gypfiles which are not at the
# project root incorrectly.
# cf. https://code.google.com/p/gyp/issues/detail?id=460
if test.format == 'xcode-ninja':
  test.skip_test()

test.run_gyp('subdir/action-rule-hash.gyp')
test.build('subdir/action-rule-hash.gyp', test.ALL)
test.up_to_date('subdir/action-rule-hash.gyp')

# Verify that everything is still up-to-date when we re-invoke gyp from a
# different directory.
test.run_gyp('action-rule-hash.gyp', '--depth=../', chdir='subdir')
test.up_to_date('subdir/action-rule-hash.gyp')

test.pass_test()



#!/usr/bin/env python
# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

from __future__ import print_function
import sys
print("This will go to stderr", file=sys.stderr)



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies library_dirs (in link_settings) are properly found.
"""

import sys

import TestGyp

test = TestGyp.TestGyp()

lib_dir = test.tempdir('secret_location')

test.run_gyp('test.gyp',
             '-D', 'abs_path_to_secret_library_location={0}'.format(lib_dir),
             chdir='subdir')

# Must build each target independently, since they are not in each others'
# 'dependencies' (test.ALL does NOT work here for some builders, and in any case
# would not ensure the correct ordering).
test.build('test.gyp', 'mylib', chdir='subdir')
test.build('test.gyp', 'libraries-search-path-test', chdir='subdir')

expect = """Hello world
"""
test.run_built_executable(
    'libraries-search-path-test', chdir='subdir', stdout=expect)

if sys.platform in ('win32', 'cygwin'):
  test.run_gyp('test-win.gyp',
               '-D',
               'abs_path_to_secret_library_location={0}'.format(lib_dir),
               chdir='subdir')

  test.build('test.gyp', 'mylib', chdir='subdir')
  test.build('test-win.gyp',
             'libraries-search-path-test-lib-suffix',
             chdir='subdir')

  test.run_built_executable(
        'libraries-search-path-test-lib-suffix', chdir='subdir', stdout=expect)


test.pass_test()
test.cleanup()



#!/usr/bin/env python

# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

""" Verifies that stdout and stderr from rules get logged in the build's
stdout."""

import sys
import TestGyp

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs'])

  test.run_gyp('rules-stdout-stderr.gyp')
  test.build('rules-stdout-stderr.gyp', test.ALL)

  expected_stdout_lines = [
    'testing stdout',
    'This will go to stdout',

    # Note: stderr output from rules will go to the build's stdout.
    'testing stderr',
    'This will go to stderr',
  ]
  test.must_contain_all_lines(test.stdout(), expected_stdout_lines)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2017 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies the use of linker flags in environment variables.

In this test, gyp and build both run in same local environment.
"""

import TestGyp

import re
import subprocess
import sys

FORMATS = ('make', 'ninja')

if sys.platform.startswith('linux'):
  test = TestGyp.TestGyp(formats=FORMATS)

  CHDIR = 'ldflags-from-environment'
  with TestGyp.LocalEnv({'LDFLAGS': '-Wl,--dynamic-linker=/target',
                         'LDFLAGS_host': '-Wl,--dynamic-linker=/host',
                         'GYP_CROSSCOMPILE': '1'}):
    test.run_gyp('test.gyp', chdir=CHDIR)
    test.build('test.gyp', chdir=CHDIR)

  def GetDynamicLinker(p):
    p = test.built_file_path(p, chdir=CHDIR)
    r = re.compile(r'\[Requesting program interpreter: ([^\]]+)\]')
    proc = subprocess.Popen(['readelf', '-l', p], stdout=subprocess.PIPE)
    o = proc.communicate()[0].decode('utf-8')
    assert not proc.returncode
    return r.search(o).group(1)

  if GetDynamicLinker('ldflags') != '/target':
    test.fail_test()

  if GetDynamicLinker('ldflags_host') != '/host':
    test.fail_test()

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2015 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Test that empty variable names don't cause infinite loops.
"""

import os

import TestGyp

test = TestGyp.TestGyp()

test.run_gyp('empty.gyp')

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2013 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that the default STRIP_STYLEs match between different generators.
"""

from __future__ import print_function

import TestGyp

import re
import subprocess
import sys
import time

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  CHDIR='strip'
  test.run_gyp('test-defaults.gyp', chdir=CHDIR)

  test.build('test-defaults.gyp', test.ALL, chdir=CHDIR)

  # Lightweight check if stripping was done.
  def OutPath(s):
    return test.built_file_path(s, chdir=CHDIR)

  def CheckNsyms(p, o_expected):
    proc = subprocess.Popen(['nm', '-aU', p], stdout=subprocess.PIPE)
    o = proc.communicate()[0].decode('utf-8')

    # Filter out mysterious "00 0000   OPT radr://5614542" symbol which
    # is apparently only printed on the bots (older toolchain?).
    # Yes, "radr", not "rdar".
    o = ''.join(filter(lambda s: 'radr://5614542' not in s, o.splitlines(True)))

    o = o.replace('A', 'T')
    o = re.sub(r'^[a-fA-F0-9]+', 'XXXXXXXX', o, flags=re.MULTILINE)
    assert not proc.returncode
    if o != o_expected:
      print('Stripping: Expected symbols """\n%s""", got """\n%s"""' % (
          o_expected, o))
      test.fail_test()

  CheckNsyms(OutPath('libsingle_dylib.dylib'),
"""\
XXXXXXXX S _ci
XXXXXXXX S _i
XXXXXXXX T _the_function
XXXXXXXX t _the_hidden_function
XXXXXXXX T _the_used_function
XXXXXXXX T _the_visible_function
""")
  CheckNsyms(OutPath('single_so.so'),
"""\
XXXXXXXX S _ci
XXXXXXXX S _i
XXXXXXXX T _the_function
XXXXXXXX t _the_hidden_function
XXXXXXXX T _the_used_function
XXXXXXXX T _the_visible_function
""")
  CheckNsyms(OutPath('single_exe'),
"""\
XXXXXXXX T __mh_execute_header
""")

  CheckNsyms(test.built_file_path(
      'bundle_dylib.framework/Versions/A/bundle_dylib', chdir=CHDIR),
"""\
XXXXXXXX S _ci
XXXXXXXX S _i
XXXXXXXX T _the_function
XXXXXXXX t _the_hidden_function
XXXXXXXX T _the_used_function
XXXXXXXX T _the_visible_function
""")
  CheckNsyms(test.built_file_path(
      'bundle_so.bundle/Contents/MacOS/bundle_so', chdir=CHDIR),
"""\
XXXXXXXX S _ci
XXXXXXXX S _i
XXXXXXXX T _the_function
XXXXXXXX T _the_used_function
XXXXXXXX T _the_visible_function
""")
  CheckNsyms(test.built_file_path(
      'bundle_exe.app/Contents/MacOS/bundle_exe', chdir=CHDIR),
"""\
XXXXXXXX T __mh_execute_header
""")

  test.pass_test()
